proc sc_ase_job {args} {
	upvar job_logdir job_logdir
	#
	set bamfile {}
	set groupmethod scsorter
	set groupfile {}
	set readassignmentsfile {}
	set pb_file {}
	cg_options sc_ase args {
		-bamfile {set bamfile $value}
		-groupmethod {set groupmethod $value}
		-groupfile {set groupfile $value}
		-readassignmentsfile {set readassignmentsfile $value}
		-pb_file {set pb_file $value}
	} {sampledir varfile resultfile} 3 3
	set sampledir [file_absolute $sampledir]
	set sample [file tail $sampledir]
	set varfile [file_absolute $varfile]
	set resultfile [file_absolute $resultfile]
	if {$bamfile eq ""} {
		set bamfile [glob $sampledir/map-*.bam $sampledir/map-*.cram]
	}
	if {$groupfile eq ""} {
		set groupfile [gzfile $sampledir/sc_group-$groupmethod-*.tsv]
	}
	if {$pb_file eq ""} {
		set pb_file [lindex [list_lremove [gzfiles $sampledir/pb_isoform_counts-$groupmethod-*.tsv] [gzfiles $sampledir/pb_isoform_counts-$groupmethod-*.colinfo.tsv]] 0]
	}
	if {$readassignmentsfile eq ""} {
		set readassignmentsfile [gzfile read_assignments-*.tsv]
	}

	set strictpct 90
	# groupfile
	puts "read group data from $groupfile"
	unset -nocomplain groupa
	array set groupa [split [string trim [cg select -f {cell group} $groupfile]] \n\t]
	#
	puts "Opening varfile $varfile"
	catch {close $f}	;	catch {close $fr}
	set f [gzopen $varfile]
	set header [tsv_open $f]
	set poss [tsv_basicfields $header 6 0]
	set o [wgzopen $resultfile]
	set outheader {chromosome begin end type ref alt sample transcript geneid group genotype}
	lappend outheader counts_weighed counts_unique counts_strict counts_aweighed counts_aunique counts_astrict
	puts $o [join $outheader \t]
	while 1 {
		if {[gets $f line] == -1} break
		set varline [list_sub [split $line \t] $poss]
		puts "checking variant: $varline"
		foreach {chromosome begin end type ref alt} $varline break
		set varline [join $varline \t]
		if {$type ne "snp"} {
			puts stderr "skipping $chromosome:$begin-$end:$type:$ref>$alt: only snps supported"
		}
		# pileup
		puts "mpileup from $bamfile"
		set temp [catch_exec samtools mpileup -r $chromosome:$end-$end --output-QNAME $bamfile]
		set temp [lindex [split $temp \n] 0]
		foreach {chr pos pref nrreads bases qualities readids} [split $temp \t] break
		# Forward	Reverse	Meaning
		# . dot	, comma	Base matches the reference base
		# ACGTN	acgtn	Base is a mismatch to the reference base
		# >	<	Reference skip (due to CIGAR "N")
		# *	*/#	Deletion of the reference base (CIGAR "D")
		unset -nocomplain readgenoa
		foreach base [split $bases ""] readid [split $readids ,] {
			if {$base in ". ,"} {set base ref}
			set base [string toupper $base]
			set readgenoa($readid) $base
		}
		# readassignments
		puts "extract readassignments region from $readassignmentsfile"
		set tempfile [tempfile].tsv
		set temp [split [cg select -q region(\"$chromosome:$begin-$end\") -f {transcript begin end} $pb_file] \n]
		set temp [lrange $temp 1 end]
		set bmin [min [list_subindex $temp 1]]
		set emax [max [list_subindex $temp 2]]
		set isoforms [list_subindex $temp 0]
		cg select -q region(\"$chromosome:$bmin-$emax\") $readassignmentsfile \
			| cg select -q "\$isoform_id in \"$isoforms\"" \
			> $tempfile
		puts "parse readassignments and count"
		set fr [open $tempfile]
		set header [tsv_open $fr]
		set rposs [list_cor $header {read_id cellbarcode isoform_id gene_id ambiguity covered_pct polya umi umicount gambiguity assignment_type inconsistency}]
		unset -nocomplain tcounta
		unset -nocomplain transcriptsa
		unset -nocomplain a
		while 1 {
			if {[gets $fr rline] == -1} break
			foreach {
				read_id cellbarcode isoform_id gene_id ambiguity covered_pct polya umi umicount gambiguity assignment_type inconsistency
			} [list_sub [split $rline \t] $rposs] break
			# putsvars read_id cellbarcode isoform_id gene_id ambiguity covered_pct polya umi umicount gambiguity assignment_type inconsistency
			if {$isoform_id eq "."} continue
			if {$inconsistency >= 2} continue
			#
			if {[info exists groupa($cellbarcode)]} {
				set group $groupa($cellbarcode)
			} else {
				set group unassigned
			}
			if {[info exists readgenoa($read_id)]} {
				set genotype $readgenoa($read_id)
			} else {
				set genotype ?
			}
			foreach {ambigcount gambigcount} [iso_isoquant_sc_ambigcount $ambiguity $gambiguity $umicount] break
			set iso $isoform_id\t$gene_id\t$group\t$genotype
			iso_isoquant_add_tcounts tcounta $iso $assignment_type $ambiguity $polya $covered_pct $strictpct $ambigcount $umicount
			set a($iso) 1
			set transcriptsa($isoform_id) $gene_id
		}
		gzclose $fr
		puts "write results to $resultfile"
		foreach iso [bsort [array names a]] {
			set resultline $varline\t$sample\t$iso
			foreach id {t u s a au as} {
				if {![info exists tcounta($iso,$id)]} {
					append resultline \t0
				} elseif {![string is int $tcounta($iso,$id)]} {
					if {$tcounta($iso,$id) > 0} {set output 1}
					append resultline \t$tcounta($iso,$id)
				} else {
					if {$tcounta($iso,$id) > 0} {set output 1}
					append resultline \t$tcounta($iso,$id)
				}
			}
			puts $o $resultline
		}
		flush $o
	}
	gzclose $o
	gzclose $f
}

proc cg_sc_ase {args} {
	set args [job_init {*}$args]
	sc_ase_job {*}$args
	job_wait
}
