#!/bin/sh
# the next line restarts using tclsh \
exec tclsh8.0 "$0" "$@"

package require Gendb
package require BioDesc
package require Extral

set mode write
set i 0
cmd_args [info script] {
	-append {switch "append to destination file"}
	-overwrite {switch "overwrite destination file"}
	-log {channel "channel on which to write logs"}
	-ftkey {any "value of ftkey in new features (dcse formats), eg. rRNA"}
	-type {any "feature type (dcse formats), eg. LSU"}
} {file ?...? resultfile} $argv
set files $file
if [info exists args] {eval lappend files $args}
if [info exists opt(-append)] {
	set mode append
} elseif [info exists opt(-overwrite)] {
	set mode overwrite
}
set ftkey [get opt(-ftkey) DCSEref]
set type [get opt(-type) ?]
set log [get opt(-log) ""]

if {![string length $resultfile]||![llength $files]} {
	puts stderr "Format is \"[info script] ?options? file ?file ...? resultfile\""
	exit 1
}

if [file exists $resultfile] {
	if [string_equal $mode overwrite] {
		file delete $resultfile
		seqdb_create $resultfile
	} elseif ![string_equal $mode append] {
		puts stderr "file \"$resultfile\" exists"
		exit 1
	}
} else {
	seqdb_create $resultfile
}
seqdb_open $resultfile db
# Load into database
foreach file $files {
	puts "Converting file \"$file\""
	set filetype [bio_filetype $file]
	switch $filetype {
		dcseref {
			conv_dcseref_seqdb -ftkey $ftkey -type $type -log $log db $file
		}
		dcseali {
			conv_dcseali_seqdb -ftkey $ftkey -type $type -log $log db $file
		}
		embl {
			conv_embl_seqdb -log stdout db $file
		}
		gbk {
			conv_gbk_seqdb -log stdout db $file
		}
		default {
			error "Cannot add file in $type format to database"
		}
	}
}
db close
