Biodb dataclass object seq

Biodb dataclass_create seq {
	$object createclass object seq {
		"db" varchar(10)
		"name" varchar(100)
		"event" integer references "event"("id") on update cascade on delete cascade
		"type" char(1)
		"acc" varchar(15)
		"version" integer
		"oldacc" varchar(1000)
		"date" date
		"release" integer
		"update" date
		"update_release" integer
		"update_version" integer
		"division" varchar(10)
		"organism" integer references "organism"("id") on update cascade on delete cascade
		"strain" varchar(100)
		"tissue_type" varchar(100)
		"organel" varchar(4)
		"def" varchar(1000)
		"length" integer
		"time" timestamp
		"nid" varchar(100)
		"comment" varchar(5000)
		"confidential" char(1)
	}
	$object index seq acc version division 
	$object createtable seq_part {
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"pos" integer
		"compressed" char(1)
		"sequence" varchar(32000)
	}
	$object index seq_part pos
	$object createtable seq_attr {
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"key" varchar(20)
		"value" varchar(1000)
	}
	$object index seq_attr key
	$object createtable seq_keywords {
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"keyword" varchar(50)
	}
	$object createtable seq_xref {
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"database" varchar(20)
		"acc" varchar(20)
		"name" varchar(20)
	}
	$object createtable seq_ref {
		"seq" integer references "seq"("id") on update cascade on delete cascade
		"refnum" integer
		"refstart" integer
		"refend" integer
		"ref" integer references "ref"("id") on update cascade on delete cascade
	}
	$object index seq_ref refnum
}

Biodb datamethod seq sequence {option id args} {
	private $object db segmentsize
	set data [$db exec -flat [subst {select "id","length" from "$dataclass" where "id" = ?}] $numid]
	if {![llength $data]} {
		error "sequence with id \"$id\" cannot be found"
	}
	set len [llength $args]
	switch -- $option {
		get {
			set size [lindex $data 1]
			set start 0
			set end [expr {$size-1}]
			if {$len > 2} {
				error "wrong # args: should be \"$object sequence get id ?start? ?end?\""
			}
			foreach {start end} $args break
			if {![string length $start] || ($start < 0)} {set start 0}
			if {![string length $end] || ($end > [expr {$size-1}])} {set end [expr {$size-1}]}
			set firstpart [expr {$start/$segmentsize}]
			set lastpart [expr {$end/$segmentsize}]
			set seq {}
			for {set partpos $firstpart} {$partpos <= $lastpart} {incr partpos} {
				foreach {compressed data} [$db exec -flat {
					select "compressed","sequence" from "seq_part"
					where "seq" = ? and "pos" = ?
				} $numid $partpos] break
				if {$compressed} {
					append seq [$object uncompress $data]
				} else {
					append seq $data
				}
			}
			set pstart [expr {$start-$firstpart*$segmentsize}]
			set pend [expr {$end-$firstpart*$segmentsize}]
			return [string range $seq $pstart $pend]
		}
		set {
			if {$len != 1} {
				error "wrong # args: should be \"$object sequence set id sequencedata\""
			}
			set seq [lindex $args 0]
			set seqlen [string length $seq]
			$db begin
			if {[catch {
				$db exec {delete from "seq_part" where "seq" = ?} $numid
				$db exec {update "seq" set "length" = ? where "id" = ?} $seqlen $numid
				set partpos 0 
				for {set pos 0} {$pos < $seqlen} {incr pos $segmentsize} {
					set part [string range $seq $pos [expr {$pos+$segmentsize-1}]]
					if {[string length $part] > 200} {
						set part [$object compress $part]
						set compressed 1
					} else {
						set compressed 0
					}
					$db exec {
						insert into "seq_part" ("seq","pos","compressed","sequence")
						values (?,?,?,?)
					} $numid $partpos $compressed $part
					incr partpos
				}
			} result]} {
				$db rollback
				return -code error -errorinfo $::errorInfo $result
			} else {
				$db commit
			}
		}
		append {
			if {$len != 1} {
				error "wrong # args: should be \"$object sequence append id sequencedata\""
			}
			set olength [lindex $data 1]
			set appendseq [lindex $args 0]
			set partpos [expr {$olength/$segmentsize}]
			if {[expr {$olength % $segmentsize}]} {
				foreach {compressed data} [$db exec -flat {
					select "compressed","sequence" from "seq_part"
					where "seq" = ? and "pos" = ?
				} $numid $partpos] break
				if {$compressed} {
					set seq [$object uncompress $data]
				} else {
					set seq $data
				}
			} else {
				set seq ""
			}
			append seq $appendseq
			$db begin
			if {[catch {
				$db exec {delete from "seq_part" where "seq" = ? and "pos" = ?} $numid $partpos
				$db exec {update "seq" set "length" = ? where "id" = ?} [expr {$olength + [string length $appendseq]}] $numid
				set seqlen [string length $seq]
				for {set pos 0} {$pos < $seqlen} {incr pos $segmentsize} {
					set part [string range $seq $pos [expr {$pos+$segmentsize-1}]]
					if {[string length $part] > 200} {
						set part [$object compress $part]
						set compressed 1
					} else {
						set compressed 0
					}
					$db exec {
						insert into "seq_part" ("seq","pos","compressed","sequence")
						values (?,?,?,?)
					} $numid $partpos $compressed $part
					incr partpos
				}
			} result]} {
				$db rollback
				return -code error -errorinfo $::errorInfo $result
			} else {
				$db commit
			}
		}
		dump {
			if {($len < 1) || ($len > 3)} {
				error "wrong # args: should be \"$object sequence dump id channel ?start? ?end?\""
			}
			set size [lindex $data 1]
			set start 0
			set end [expr {$size-1}]
			foreach {f start end} $args break
			if {![string length $start] || ($start < 0)} {set start 0}
			if {![string length $end] || ($end > [expr {$size-1}])} {set end [expr {$size-1}]}
			set firstpart [expr {$start/$segmentsize}]
			set lastpart [expr {$end/$segmentsize}]
			set seq {}
			set pstart [expr {$start-$firstpart*$segmentsize}]
			for {set partpos $firstpart} {$partpos <= $lastpart} {incr partpos} {
				foreach {compressed data} [$db exec -flat {
					select "compressed","sequence" from "seq_part"
					where "seq" = ? and "pos" = ?
				} $numid $partpos] break
				if {$compressed} {
					set part [$object uncompress $data]
				} else {
					set part $data
				}
				if {$partpos == $lastpart} {
					set part [string range $part 0 [expr {$end-$partpos*$segmentsize}]]
				}
				if {$pstart} {
					set part [string range $part $pstart end]
					set pstart 0
				}
				puts -nonewline $f $part
			}
		}
		default {
			error "unknown option \"$option\""
		}
	}
}

Biodb datamethod seq delete {id args} {
	private $object db
	$db exec {
		delete from "seq_part" where "seq" = ?
	} $numid
	$db exec [subst {delete from "$dataclass" where "id" = ?}] $numid
}
