package require Class
Class subclass bio::BioMap

bio::BioMap method init {args} {
	private $object options redraw zoom
	super init
	array set options {
		-canvas {} -tag {} -area {20m 20m 277m 190m}
		-start {}
		-end {}
		-biodb {}
		-sequence {}
		-step {}
		-axisfont {helvetica 10}
		-yspacing 40
		-barwidth 10
		-labelfont {helvetica 3}
		-linewidth 0.2
		-baselinewidth 0.2
		-maxshift 2
	}
	set redraw 0
	set zoom 1
	set options(-tag) "map:$object"
	eval $object configure $args
	return $object
}

#doc {Map command destroy} cmd {
#pathname destroy 
#} descr {
#}
bio::BioMap method destroy {} {
	private $object options dbclone
	::Classy::busy
	if {[info exists dbclone]} {
		$dbclone close
	}
	catch {$options(-canvas) delete $options(-tag)}
	update idletasks
	::Classy::busy remove
}

# ------------------------------------------------------------------
#  Methods
# ------------------------------------------------------------------

#doc {Map command configure} cmd {
#pathname configure ?option? ?value? ?option value ...?
#} descr {
# change the Map configuration options
#}
bio::BioMap method configure {args} {
	private $object options redraw dbclone
	set len [llength $args]
	if {$len == 0} {
		return [array get options]
	} elseif {$len == 1} {
		return $options([lindex $args 0])
	} else {
		foreach {option value} $args {
			if ![info exists options($option)] {
				error "unknown option \"$option\""
			}
			set previous $options($option)
			set options($option) $value
			switch -- $option {
				-canvas {
					private $object options
					if {[string_equal $value ""]} {
						$previous delete $options(-tag)
						set redraw 0
						return
					}
					if {[string_equal $options(-tag) {}]} {
						set options(-tag) map::$object
					}
					update idletasks
					if {[string_equal $options(-area) {}]} {
						set options(-area) [list 25 10 [expr [winfo width $value]-10] [expr [winfo height $value]-25]]
					}
					::Classy::todo $object redraw
				}
				-tag {
					if {[string_equal $options(-tag) {}]} {
						set options(-tag) "map:$object"
					}
					catch {$options(canvas) delete $previous}
					::Classy::todo $object redraw
				}
				-biodb {
					if {[info exists dbclone]} {
						$dbclone close
						unset dbclone
					}
					if {![string_equal $options(-biodb) {}]} {
						set db [$options(-biodb) db]
						set dbclone [$db clone]
					}
					::Classy::todo $object redraw
				}
				default {
					::Classy::todo $object redraw
				}
			}
		}
	}
	if {![string_equal $options(-biodb) ""] && ![string_equal $options(-sequence) ""]} {
		if {![string_equal $options(-canvas) ""]} {
			set redraw 1
		} else {
			set redraw 0
		}
	} else {
		set redraw 0
	}
}

bio::BioMap method _settings {} {
	private $object options redraw wstart wend step xs xe xscale ys ye len zoom redraw
	if {!$redraw} return
	set biodb $options(-biodb)
	set sequence $options(-sequence)
	set len [$biodb get $sequence length]
	if {[string_equal $options(-start) ""]} {set wstart 0} else {set wstart $options(-start)}
	if {[string_equal $options(-end) ""]} {set wend $len} else {set wend $options(-end)}
	set size [expr {$wend-$wstart}]
	if {![string_equal $options(-step) ""]} {
		set step $options(-step)
	} else {
		set temp [expr {$size/10}]
		set l [string length $temp]
		set i [string index $temp 0]
		if {$i<2} {
			set step 1
		} elseif {$i<7} {
			set step 5
		} else {
			set step 10
		}
		for {set i 1} {$i < $l} {incr i} {
			append step 0
		}
	}
	foreach {xs ys xe ye} $options(-area) break
	set w [get options(-canvas) .]
#	set xs [expr {[winfo fpixels $w $xs]/[winfo fpixels $w 1m]}]
#	set xe [expr {[winfo fpixels $w $xe]/[winfo fpixels $w 1m]}]
#	set ys [expr {[winfo fpixels $w $ys]/[winfo fpixels $w 1m]}]
	set xs [expr {[winfo fpixels $w $xs]*$zoom}]
	set xe [expr {[winfo fpixels $w $xe]*$zoom}]
	set ys [expr {[winfo fpixels $w $ys]*$zoom}]
	set ye [expr {[winfo fpixels $w $ye]*$zoom}]
	set xscale [expr {($xe-$xs)/$size}]
	if {$wend > $len} {
		set xe [expr {$xs+$xscale*($len-$wstart)}]
	}
}


proc ::bio::zoomfont {font zoom} {
	set size [expr {round([lindex $font 1]*$zoom)}]
	return [lreplace $font 1 1 $size]
}

bio::BioMap method zoom {newzoom {scroll 0}} {
	private $object options zoom redraw
	if {!$redraw} return
	set w $options(-canvas)
	if {$scroll} {
		set factor [expr {double($newzoom)/$zoom}]
		foreach c [$w cget -scrollregion] {
			set c [winfo fpixels $w $c]
			lappend scrollregion [expr {double($c)*$factor}]
		}
		$w configure -scrollregion $scrollregion
	}
	set zoom $newzoom
	::Classy::todo $object redraw
}

bio::BioMap method draw_axis {} {
	private $object options redraw wstart wend step xs xe xscale ys ye len zoom
	if {!$redraw} return
	set w $options(-canvas)
	set tag $options(-tag)
	set font [bio::zoomfont $options(-axisfont) $zoom]
	set ticky [expr {$ys-4}]
	set xstep [expr {$step*$xscale}]
	set x $xs
	for {set num $wstart} {$num <= $wend} {incr num $step} {
		if {$x > [expr {$xe+0.01}]} {
			$w create text [expr {$xe+1}] [expr {$ys}] -text $len -anchor w \
				-tags [list $tag axisnum] -font $font
			break
		}
		$w create line $x $ys $x $ye -fill gray -tags [list $tag axisbar]
		$w create line $x $ys $x $ticky -tags [list $tag axistick]
		$w create text $x [expr {$ys-6}] -text $num -anchor sw -justify center \
			-tags [list $tag axisnum] -font $font
		set x [expr {$x+$xstep}]
	}
#	$w create line $xs $ys $xe $ys -tags [list $tag axis]
	$w create rectangle $xs $ys $xe $ye -tags [list $tag axisbox]
}

bio::BioMap method draw_filter {name opts} {
	private $object options wstart wend xs xe xscale ys ye ypos zoom redraw dbclone
	if {!$redraw} return
	set biodb $options(-biodb)
	set db [$biodb db]
	array set opt $opts
	set w $options(-canvas)
	set table $opt(-table)
	set query [get opt(-query) {}]
	set uselabels [get opt(-labels) 1]
	set label $name
	set tag $options(-tag)
	set legend [get opt(-legend) $name]
	set labelfont [get opt(-labelfont) $options(-labelfont)]
	set linewidth [get opt(-linewidth) $options(-linewidth)]
	set baselinewidth [get opt(-baselinewidth) $options(-baselinewidth)]
	set width [get opt(-width) $options(-barwidth)]
	set maxshift [get opt(-maxshift) $options(-maxshift)]
	if {[info exists opt(-ypos)]} {
		set ybase $opt(-ypos)
	} else {
		while {[info exists ypos($ypos())]} {
			incr ypos() $options(-yspacing)
		}
		set ybase $ypos()
	}
	set ypos($ybase) 1
	set ybase [expr {$ys+$ybase*$zoom}]
	set fill [get opt(-fill) blue]
	set outline [get opt(-outline) black]
	set sequence $options(-sequence)
	if {[info exists opt(-itemcommand)]} {
		proc ::bio::itemcommand {biodb ft start end} "upvar data data\nupvar use use\n$opt(-itemcommand)"
		set itemcommand 1
	} else {
		set itemcommand 0
	}
	# query
	if {[string length $query]} {
		set doquery [subst {
			select * from "$table"
			where ($query) and "seq" = [lindex $sequence 1] and "start" < $wend and "end" > $wstart
			order by "start"
		}]
	} else {
		set doquery [subst {
			select * from "$table"
			where "seq" = [lindex $sequence 1] and "start" < $wend and "end" > $wstart
			order by "start"
		}]
	}
	puts \n-----------------\n$doquery\n-----------------
	puts "querytime: [time {set ldata [$dbclone exec -flat $doquery]}]"
	# draw
	set y [expr {$ybase-$width*$zoom}]
	set yc [expr {$ybase+$width*$zoom}]
	$w create line $xs $ybase $xe $ybase -tags [list $tag baseline $name] -fill gray -width $baselinewidth
	$w create text [expr {$xs-5}] $ybase -tags [list $tag legend $name] -text $legend -anchor e
	set lastx(0) -1
	set lastx(1) -1
	set shifted(0) 0
	set shifted(1) 0
	set num 0
	set dfields {}
	foreach field [$db fields $table] {
		lappend dfields data($field)
	}
	foreach $dfields $ldata {
		incr num
		set ft $data(id)
		set ftstart $data(start)
		set ftend $data(end)
		set complement $data(complement)
		if {[string equal $complement ""]} {set complement 0}
		set composite $data(composite)
		if {[string equal $composite ""]} {set composite 0}
		catch {unset use}
		if {$itemcommand} {
			::bio::itemcommand $biodb $ft $ftstart $ftend
		}
		if {$uselabels} {
			set f_labelfont [::bio::zoomfont [get use(font) $labelfont] $zoom]
			if {$shifted($complement) >= $maxshift} {
				set lastx($complement) -1
			}
		}
		set f_linewidth [expr {[get use(linewidth) $linewidth]*$zoom}]
		set xstart [expr {$xs+$xscale*($ftstart-$wstart)}]
		if {$xstart < $xs} {set xstart $xs}
		set xend [expr {$xs+$xscale*($ftend+1-$wstart)}]
		if {$xend > $xe} {set xend $xe}
		if {!$composite} {
			if {!$complement} {
				if {![info exists use(width)]} {
					set y2 $y
				} else {
					set y2 [expr {$ybase-$use(width)*$zoom}]
				}
				set t +
				set ly [expr {$y2-1}]
				set lastypos 1
				set anchor sw
			} else {
				if {![info exists use(width)]} {
					set y2 $yc
				} else {
					set y2 [expr {$ybase+$use(width)*$zoom}]
				}
				set t -
				set ly [expr {$y2+1}]
				set lastypos 3
				set anchor nw
			}
			$w create rectangle $xstart $ybase $xend $y2 \
				-fill [get use(fill) $fill] -outline [get use(outline) $outline] \
				-width $f_linewidth \
				-tags [list $tag feature $name [list $table $ft] $t]
			if {$uselabels} {
				if {$xstart < $lastx($complement)} {
					incr shifted($complement)
					set ly $lasty($complement)
				} else {
					set shifted($complement) 0
				}
				set id [$w create text $xstart $ly \
					-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
					-tags [list $tag label $name [list $table $ft] $t]]
				set bbox [$w bbox $id]
				set lasty($complement) [lindex $bbox $lastypos]
				if {$shifted($complement) == 0} {
					set lastx($complement) [lindex $bbox 2]
				}
			}
		} else {
			if {!$complement} {
				set uy $y
				set t +
				set ly [expr {$y-1}]
				set anchor s
			} else {
				set uy $yc
				set t -
				set ly [expr {$yc+1}]
				set anchor n
			}
			$w create line $xstart $uy $xend $uy \
				-fill [get use(fill) $fill] \
				-width $f_linewidth \
				-tags [list $tag combofeature $name [list $table $ft] $t]
			set xpos [expr {$xstart + ($xend-$xstart)/2}]
			if {$uselabels} {
				set id [$w create text $xpos $ly \
					-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
					-tags [list $tag label $name [list $table $ft] $t]]
			}
		}
	}
	eval $w raise combofeature all
	return $num
}

bio::BioMap method page {size} {
	private $object options
	set biodb $options(-biodb)
	set sequence $options(-sequence)
	set maxlen [$biodb get $sequence length]
	set start $options(-start)
	set end $options(-end)
	set len [expr {$end - $start}]
	set shift [expr {$size*$len}]
	incr start $shift
	if {$start > $maxlen} {return}
	incr end $shift
	$object configure -start $start -end $end
}

bio::BioMap method redraw {} {
	private $object options redraw filter filterorder ypos
	if {!$redraw} return
	$object _settings
	set w $options(-canvas)
	$w delete $options(-tag)
	$object draw_axis
	catch {unset ypos}
	set ypos() $options(-yspacing)
	set names [list_union [get filterorder ""] [array names filter]]
	foreach name $names {
		if {[info exists filter($name)]} {
			puts -nonewline "filter: $name: "
			set time [time {set num [$object draw_filter $name $filter($name)]}]
			puts $num
			puts $time
		}
	}
}

bio::BioMap method filter {option args} {
	private $object options filter filterorder
	switch $option {
		add {
			set filter([list_shift args]) $args
		}
		delete {
			foreach name $args {
				catch {unset filter($name)}
			}
		}
		order {
			set filterorder [lindex $args 0]
		}
		list {
			return [list_union [get filterorder ""] [array names filter]]
		}
	}
	::Classy::todo $object redraw	
}

bio::BioMap method printdialog {} {
	private $object options redraw
	if {!$redraw} return
	set w $options(-canvas)
	set page [$w cget -scrollregion]
	set width [lindex $page 2]
	set height [lindex $page 3]
	if {$width < $height} {
		set portrait 1
	} else {
		set portrait 0
	}
	if [winfo exists .classy__.printdialog] {
		destroy .classy__.printdialog
	}
	Classy_printdialog .classy__.printdialog -portrait 0 -colormode color \
		-papersize [list $width $height] \
		-canvas $w -command {}
}

bio::BioMap method print {args} {
	private $object options redraw
	if {!$redraw} return
	set w $options(-canvas)
	set page [$w cget -scrollregion]
	set width [lindex $page 2]
	set height [lindex $page 3]
	array set print [list -colormode color -printwidth $width -printheight $height]
	array set print $args
	eval ::Classy::printcanvas $w [array get print]
}

bio::BioMap method draw_filter {name opts} {
#putsvars name opts
	private $object options wstart wend xs xe xscale ys ye ypos zoom redraw dbclone
	if {!$redraw} return
	set biodb $options(-biodb)
	set db [$biodb db]
	array set opt $opts
	set w $options(-canvas)
	set table $opt(-table)
	set query [get opt(-query) {}]
	set uselabels [get opt(-labels) 1]
	set label $name
	set tag $options(-tag)
	set legend [get opt(-legend) $name]
	set labelfont [get opt(-labelfont) $options(-labelfont)]
	set linewidth [get opt(-linewidth) $options(-linewidth)]
	set baselinewidth [get opt(-baselinewidth) $options(-baselinewidth)]
	set width [get opt(-width) $options(-barwidth)]
	set maxshift [get opt(-maxshift) $options(-maxshift)]
	if {[info exists opt(-ypos)]} {
		set ybase $opt(-ypos)
	} else {
		while {[info exists ypos($ypos())]} {
			incr ypos() $options(-yspacing)
		}
		set ybase $ypos()
	}
	set ypos($ybase) 1
	set ybase [expr {$ys+$ybase*$zoom}]
	set fill [get opt(-fill) blue]
	set outline [get opt(-outline) black]
	set sequence $options(-sequence)
	if {[info exists opt(-itemcommand)]} {
		proc ::bio::itemcommand {biodb ft start end} "upvar data data\nupvar use use\n$opt(-itemcommand)"
		set itemcommand 1
	} else {
		set itemcommand 0
	}
	# query
	if {[string length $query]} {
		set doquery [subst {
			select * from "$table"
			where ($query) and "seq" = [lindex $sequence 1] and "start" < $wend and "end" > $wstart
			order by "start"
		}]
	} else {
		set doquery [subst {
			select * from "$table"
			where "seq" = [lindex $sequence 1] and "start" < $wend and "end" > $wstart
			order by "start"
		}]
	}
	puts \n-----------------\n$doquery\n-----------------
	puts "querytime: [time {set ldata [$dbclone exec -flat $doquery]}]"
	# draw
	set y [expr {$ybase-$width*$zoom}]
	set yc [expr {$ybase+$width*$zoom}]
	$w create line $xs $ybase $xe $ybase -tags [list $tag baseline $name] -fill gray -width $baselinewidth
	$w create text [expr {$xs-5}] $ybase -tags [list $tag legend $name] -text $legend -anchor e
	set lastx(0) -1
	set lastx(1) -1
	set shifted(0) 0
	set shifted(1) 0
	set num 0
	set dfields {}
	foreach field [$db fields $table] {
		lappend dfields data($field)
	}
	foreach $dfields $ldata {
		incr num
		set ft $data(id)
		set ftstart $data(start)
		set ftend $data(end)
		set complement $data(complement)
		if {[string equal $complement ""]} {set complement 0}
		set composite $data(composite)
		if {[string equal $composite ""]} {set composite 0}
		catch {unset use}
		if {$itemcommand} {
			::bio::itemcommand $biodb $ft $ftstart $ftend
		}
		if {$uselabels} {
			set f_labelfont [::bio::zoomfont [get use(font) $labelfont] $zoom]
			if {$shifted($complement) >= $maxshift} {
				set lastx($complement) -1
			}
		}
		set f_linewidth [expr {[get use(linewidth) $linewidth]*$zoom}]
		set xstart [expr {$xs+$xscale*($ftstart-$wstart)}]
		if {$xstart < $xs} {set xstart $xs}
		set xend [expr {$xs+$xscale*($ftend+1-$wstart)}]
		if {$xend > $xe} {set xend $xe}
		if {!$composite} {
			if {!$complement} {
				if {![info exists use(width)]} {
					set y2 $y
				} else {
					set y2 [expr {$ybase-$use(width)*$zoom}]
				}
				set t +
				set ly [expr {$y2-1}]
				set lastypos 1
				set anchor sw
			} else {
				if {![info exists use(width)]} {
					set y2 $yc
				} else {
					set y2 [expr {$ybase+$use(width)*$zoom}]
				}
				set t -
				set ly [expr {$y2+1}]
				set lastypos 3
				set anchor nw
			}
			$w create rectangle $xstart $ybase $xend $y2 \
				-fill [get use(fill) $fill] -outline [get use(outline) $outline] \
				-width $f_linewidth \
				-tags [list $tag feature $name [list $table $ft] $t]
			if {$uselabels} {
				if {$xstart < $lastx($complement)} {
					incr shifted($complement)
					set ly $lasty($complement)
				} else {
					set shifted($complement) 0
				}
				set id [$w create text $xstart $ly \
					-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
					-tags [list $tag label $name [list $table $ft] $t]]
				set bbox [$w bbox $id]
				set lasty($complement) [lindex $bbox $lastypos]
				if {$shifted($complement) == 0} {
					set lastx($complement) [lindex $bbox 2]
				}
			}
		} else {
			if {!$complement} {
				set uy $y
				set t +
				set ly [expr {$y-1}]
				set anchor s
			} else {
				set uy $yc
				set t -
				set ly [expr {$yc+1}]
				set anchor n
			}
			$w create line $xstart $uy $xend $uy \
				-fill [get use(fill) $fill] \
				-width $f_linewidth \
				-tags [list $tag combofeature $name [list $table $ft] $t]
			set xpos [expr {$xstart + ($xend-$xstart)/2}]
			if {$uselabels} {
				set id [$w create text $xpos $ly \
					-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
					-tags [list $tag label $name [list $table $ft] $t]]
			}
		}
	}
	eval $w raise combofeature all
	return $num
}

#bio::BioMap method show_table {{items {}}} {
#	private $object options selectdata fts fields
#	set w $options(-canvas).table
#	set biodb $options(-biodb)
#	if {[llength $items]} {
#		set fts {}
#		foreach item $items {
#			lappend fts [lindex [.c itemcget $item -tags] 3]
#		}
#		if {![llength $fts]} {
#			puts "no features selected"
#			break
#		}
#		set fts [lsort -index 0 [list_remdup $fts]]
#	}
#	set ftfields [$biodb fields ft]
#	if {![winfo exists $w]} {
#		set fields $ftfields
#		toplevel $w
#		Classy::Table $w.table \
#			-xscrollcommand [list $w.xsc set] \
#			-yscrollcommand [list $w.ysc set] \
#			-variable [privatevar $object selectdata]
#		scrollbar $w.xsc -orient horizontal -command [list $w.table xview]
#		scrollbar $w.ysc -orient vertical -command [list $w.table yview]
#		grid $w.table $w.ysc -sticky nwse -row 0
#		grid $w.xsc -sticky we -row 1
#		grid columnconfigure $w 0 -weight 1
#		grid rowconfigure $w 0 -weight 1
#		$w.table configure -titlecols 1 -titlerows 1 -cols 40 -xresize 1
#		$w.table tag configure title -bg gray
#		for {set i 1} {$i < 40} {incr i} {
#			Classy::OptionMenu $w.table.c$i -list $ftfields
#			$w.table window configure 0,$i -window $w.table.c$i -sticky we
#		}
#		set i 1
#		foreach field {ftkey type name start end complement composite score product gene protein_id hit_id hit_db hit_div hit_gi hit_acc hit_version hit_locus hit_def hit_full_length hit_start hit_end hit_complement hit_size} {
#			$w.table.c$i set $field
#			incr i
#		}
#		for {set i 1} {$i < 40} {incr i} {
#			$w.table.c$i configure -command [list $object show_table]
#		}
#	}
#	set classes [list_remdup [list_subindex $fts 0]]
#	foreach class $classes {
#		eval lappend fields [$biodb fields $class]
#	}
#	set fields [list_remdup $fields]
#	for {set i 1} {$i < 40} {incr i} {
#		$w.table.c$i configure -list $fields
#	}
#	set sfields {}
#	for {set i 1} {$i < 40} {incr i} {
#		lappend sfields [$w.table.c$i get]
#	}
#	catch {unset selectdata}
#	set db [$biodb db]
#	set row 1
#	foreach ft $fts {
#		set selectdata(0,$row) $ft
#		set line [eval {$db get $ft} $sfields]
#		# $w.table set row $row,1 $line
#		set i 1
#		foreach el $line {
#			set selectdata($i,$row) $el
#			incr i
#		}
#	}
#	$w.table _redraw
#}
#
#bio::BioMap method show_table {{items {}}} {
#	private $object options selectdata fts fields
#	set w $options(-canvas).table
#	set biodb $options(-biodb)
#	if {[llength $items]} {
#		set fts {}
#		foreach item $items {
#			set ft [lindex [.c itemcget $item -tags] 3]
#			if {[llength $ft] == 2} {
#				lappend fts $ft
#			}
#		}
#		if {![llength $fts]} {
#			puts "no features selected"
#			break
#		}
#		set fts [lsort -index 0 [list_remdup $fts]]
#	}
#	set ftfields [$biodb fields ft]
#	if {![winfo exists $w]} {
#		set fields $ftfields
#		eval lappend fields [$biodb fields blast]
#		set fields [list_remdup $fields]
#		toplevel $w
#		table $w.table \
#			-xscrollcommand [list $w.xsc set] \
#			-yscrollcommand [list $w.ysc set] \
#			-selectmode extended \
#			-variable [privatevar $object selectdata]
#		scrollbar $w.xsc -orient horizontal -command [list $w.table xview]
#		scrollbar $w.ysc -orient vertical -command [list $w.table yview]
#		grid $w.table $w.ysc -sticky nwse -row 0
#		grid $w.xsc -sticky we -row 1
#		grid columnconfigure $w 0 -weight 1
#		grid rowconfigure $w 0 -weight 1
#		$w.table configure -titlecols 1 -titlerows 1 -cols 40 -wrap 1 -anchor nw
#		for {set i 1} {$i < 40} {incr i} {
#			catch {destroy $w.table.c$i}
#			Classy::OptionMenu $w.table.c$i -list $fields
#			$w.table window configure 0,$i -window $w.table.c$i -sticky we
#		}
#		set i 1
#		foreach field {ftkey start end complement composite score hit_db hit_div hit_gi hit_acc hit_id hit_version hit_locus hit_def hit_full_length hit_start hit_end hit_complement hit_size} {
#			$w.table.c$i set $field
#			incr i
#		}
#		for {set i 1} {$i < 40} {incr i} {
#			$w.table.c$i configure -command [list $object show_table]
#		}
#	}
#	set classes [list_remdup [list_subindex $fts 0]]
#	foreach class $classes {
#		eval lappend fields [$biodb fields $class]
#	}
#	set fields [list_remdup $fields]
#	for {set i 1} {$i < 40} {incr i} {
#		$w.table.c$i configure -list $fields
#	}
#	set sfields {}
#	for {set i 1} {$i < 40} {incr i} {
#		lappend sfields [$w.table.c$i get]
#	}
#	set db [$biodb db]
#	catch {unset selectdata}
#	set row 1
#	foreach ft $fts {
#		if {![llength $ft]} continue
#		set selectdata($row,0) $ft
#		set line [eval {$db get $ft} $sfields]
#		set col 1
#		foreach value $line {
#			set selectdata($row,$col) $value
#			incr col
#		}
#		incr row
#	}
#}

bio::BioMap method show_table {{items {}}} {
	private $object options fts fields
	set w $options(-canvas).table
	set biodb $options(-biodb)
	if {![info exists fts]} {set fts {}}
	if {[llength $items]} {
		set fts {}
		foreach item $items {
			set ft [lindex [.c itemcget $item -tags] 3]
			if {[llength $ft] == 2} {
				lappend fts $ft
			}
		}
		if {![llength $fts]} {
			puts "no features selected"
		}
		set fts [lsort -index 0 [list_remdup $fts]]
	}
	set ftfields [$biodb fields ft]
	if {![winfo exists $w]} {
		set fields $ftfields
		eval lappend fields [$biodb fields blast]
		set fields [list_remdup $fields]
		toplevel $w
		table $w.table \
			-xscrollcommand [list $w.xsc set] \
			-yscrollcommand [list $w.ysc set] \
			-cache 1 \
			-selectmode extended
		scrollbar $w.xsc -orient horizontal -command [list $w.table xview]
		scrollbar $w.ysc -orient vertical -command [list $w.table yview]
		grid $w.table $w.ysc -sticky nwse -row 0
		grid $w.xsc -sticky we -row 1
		grid columnconfigure $w 0 -weight 1
		grid rowconfigure $w 0 -weight 1
		$w.table configure -titlecols 1 -titlerows 1 -cols 40 -wrap 1 -anchor nw
		for {set i 1} {$i < 40} {incr i} {
			catch {$w.table.c$i destroy}
			Classy::OptionMenu $w.table.c$i -list $fields
			$w.table window configure 0,$i -window $w.table.c$i -sticky we
		}
		set i 1
		foreach field {ftkey start end complement composite score hit_db hit_div hit_gi hit_acc hit_id hit_version hit_locus hit_def hit_full_length hit_start hit_end hit_complement hit_size} {
			$w.table.c$i set $field
			incr i
		}
		for {set i 1} {$i < 40} {incr i} {
			$w.table.c$i configure -command [list invoke {} [list $object show_table]]
		}
	}
	set classes [list_remdup [list_subindex $fts 0]]
	foreach class $classes {
		eval lappend fields [$biodb fields $class]
	}
	set fields [list_remdup $fields]
	for {set i 1} {$i < 40} {incr i} {
		$w.table.c$i configure -list $fields
	}
	set sfields {}
	for {set i 1} {$i < 40} {incr i} {
		lappend sfields [$w.table.c$i get]
	}
	set db [$biodb db]
	$w.table configure -rows [expr {[llength $fts]+1}]
	set result {}
	foreach ft $fts {
		if {![llength $ft]} continue
		set line [eval {$db get $ft} $sfields]
		list_unshift line $ft
		lappend result $line
	}
	set ::result [lsort -dictionary -index 1 $result]
	set row 1
	foreach line $::result {
		$w.table set row $row,0 $line
		incr row
	}
}

bio::BioMap method write_table {file {items {}}} {
	private $object options fts fields
	set w $options(-canvas).table
	set biodb $options(-biodb)
	if {![info exists fts]} {set fts {}}
	if {[llength $items]} {
		set fts {}
		foreach item $items {
			set ft [lindex [.c itemcget $item -tags] 3]
			if {[llength $ft] == 2} {
				lappend fts $ft
			}
		}
		if {![llength $fts]} {
			puts "no features selected"
		}
		set fts [lsort -index 0 [list_remdup $fts]]
	}
	set fields [$biodb fields ft]
	set classes [list_remdup [list_subindex $fts 0]]
	foreach class $classes {
		eval lappend fields [$biodb fields $class]
	}
	set fields [list_concat [list_common {ftkey start end complement composite score hit_db hit_div hit_gi hit_acc hit_id hit_version hit_locus hit_def hit_full_length hit_start hit_end hit_complement hit_size} $fields] $fields]
	set fields [list_remdup $fields]
	set db [$biodb db]
	set f [open $file w]
	puts $f ft\t[join $fields \t]
	foreach ft $fts {
		if {![llength $ft]} continue
		set line [eval {$db get $ft} $fields]
		list_unshift line $ft
		puts $f [join $line \t]
	}
	close $f
	puts "$file written"
}

