package require Class
Class subclass bio::Map

bio::Map method init {args} {
	private $object options
	super init
	array set options {
		-canvas {} -tag {} -area {20m 20m 277m 190m}
		-start {}
		-end {}
		-db {}
		-source {}
		-step {}
		-axisfont {helvetica 10}
		-yspacing 40
		-barwidth 10
		-labelfont {helvetica 3}
		-linewidth 0.2
		-baselinewidth 0.2
		-maxshift 2
	}
	setprivate $object redraw 0
	setprivate $object zoom 1
	set options(-tag) "map:$object"
	eval $object configure $args
	return $object
}

#doc {Map command destroy} cmd {
#pathname destroy 
#} descr {
#}
bio::Map method destroy {} {
	private $object options
	::Classy::busy
	catch {$options(-canvas) delete $options(-tag)}
	update idletasks
	::Classy::busy remove
}

# ------------------------------------------------------------------
#  Methods
# ------------------------------------------------------------------

#doc {Map command configure} cmd {
#pathname configure ?option? ?value? ?option value ...?
#} descr {
# change the Map configuration options
#}
bio::Map method configure {args} {
	private $object options redraw
	set len [llength $args]
	if {$len == 0} {
		return [array get options]
	} elseif {$len == 1} {
		return $options([lindex $args 0])
	} else {
		foreach {option value} $args {
			if ![info exists options($option)] {
				error "unknown option \"$option\""
			}
			set previous $options($option)
			set options($option) $value
			switch -- $option {
				-canvas {
					private $object options
					if {[string_equal $value ""]} {
						$previous delete $options(-tag)
						set redraw 0
						return
					}
					if {[string_equal $options(-tag) {}]} {
						set options(-tag) map::$object
					}
					update idletasks
					if {[string_equal $options(-area) {}]} {
						set options(-area) [list 25 10 [expr [winfo width $value]-10] [expr [winfo height $value]-25]]
					}
					::Classy::todo $object redraw
				}
				-tag {
					if {[string_equal $options(-tag) {}]} {
						set options(-tag) "map:$object"
					}
					catch {$options(canvas) delete $previous}
					::Classy::todo $object redraw
				}
				default {
					::Classy::todo $object redraw
				}
			}
		}
	}
	if {![string_equal $options(-db) ""] && ![string_equal $options(-source) ""]} {
		if {![string_equal $options(-canvas) ""]} {
			set redraw 1
		} else {
			set redraw 0
		}
	} else {
		set redraw 0
	}
}

bio::Map method _settings {} {
	private $object options redraw wstart wend step xs xe xscale ys ye len zoom redraw
	if {!$redraw} return
	set db $options(-db)
	set source $options(-source)
	set len [$db get $source length]
	if {[string_equal $options(-start) ""]} {set wstart 0} else {set wstart $options(-start)}
	if {[string_equal $options(-end) ""]} {set wend $len} else {set wend $options(-end)}
	set size [expr {$wend-$wstart}]
	if {![string_equal $options(-step) ""]} {
		set step $options(-step)
	} else {
		set temp [expr {$size/10}]
		set l [string length $temp]
		set i [string index $temp 0]
		if {$i<2} {
			set step 1
		} elseif {$i<7} {
			set step 5
		} else {
			set step 10
		}
		for {set i 1} {$i < $l} {incr i} {
			append step 0
		}
	}
	foreach {xs ys xe ye} $options(-area) break
	set w [get options(-canvas) .]
#	set xs [expr {[winfo fpixels $w $xs]/[winfo fpixels $w 1m]}]
#	set xe [expr {[winfo fpixels $w $xe]/[winfo fpixels $w 1m]}]
#	set ys [expr {[winfo fpixels $w $ys]/[winfo fpixels $w 1m]}]
	set xs [expr {[winfo fpixels $w $xs]*$zoom}]
	set xe [expr {[winfo fpixels $w $xe]*$zoom}]
	set ys [expr {[winfo fpixels $w $ys]*$zoom}]
	set ye [expr {[winfo fpixels $w $ye]*$zoom}]
	set xscale [expr {($xe-$xs)/$size}]
	if {$wend > $len} {
		set xe [expr {$xs+$xscale*($len-$wstart)}]
	}
}


proc ::bio::zoomfont {font zoom} {
	set size [expr {[lindex $font 1]*$zoom}]
	return [lreplace $font 1 1 $size]
}

bio::Map method zoom {newzoom {scroll 0}} {
	private $object options zoom redraw
	if {!$redraw} return
	set w $options(-canvas)
	if {$scroll} {
		set factor [expr {double($newzoom)/$zoom}]
		foreach c [$w cget -scrollregion] {
			set c [winfo fpixels $w $c]
			lappend scrollregion [expr {double($c)*$factor}]
		}
		$w configure -scrollregion $scrollregion
	}
	set zoom $newzoom
	::Classy::todo $object redraw
}

bio::Map method draw_axis {} {
	private $object options redraw wstart wend step xs xe xscale ys ye len zoom
	if {!$redraw} return
	set w $options(-canvas)
	set tag $options(-tag)
	set font [bio::zoomfont $options(-axisfont) $zoom]
	set ticky [expr {$ys-4}]
	set xstep [expr {$step*$xscale}]
	set x $xs
	for {set num $wstart} {$num <= $wend} {incr num $step} {
		if {$x > [expr {$xe+0.01}]} {
			$w create text [expr {$xe+1}] [expr {$ys}] -text $len -anchor w \
				-tags [list $tag axisnum] -font $font
			break
		}
		$w create line $x $ys $x $ye -fill gray -tags [list $tag axisbar]
		$w create line $x $ys $x $ticky -tags [list $tag axistick]
		$w create text $x [expr {$ys-6}] -text $num -anchor sw -justify center \
			-tags [list $tag axisnum] -font $font
		set x [expr {$x+$xstep}]
	}
#	$w create line $xs $ys $xe $ys -tags [list $tag axis]
	$w create rectangle $xs $ys $xe $ye -tags [list $tag axisbox]
}

bio::Map method draw_filter {name opts} {
	private $object options wstart wend xs xe xscale ys ye ypos zoom redraw
	if {!$redraw} return
	array set opt $opts
	set w $options(-canvas)
	set query $opt(-query)
	set label $opt(-query)
	set tag $options(-tag)
	set legend [get opt(-legend) $name]
	set labelfont [get opt(-labelfont) $options(-labelfont)]
	set linewidth [get opt(-linewidth) $options(-linewidth)]
	set baselinewidth [get opt(-baselinewidth) $options(-baselinewidth)]
	set width [get opt(-width) $options(-barwidth)]
	set maxshift [get opt(-maxshift) $options(-maxshift)]
	if {[info exists opt(-ypos)]} {
		set ybase $opt(-ypos)
	} else {
		while {[info exists ypos($ypos())]} {
			incr ypos() $options(-yspacing)
		}
		set ybase $ypos()
	}
	set ypos($ybase) 1
	set ybase [expr {$ys+$ybase*$zoom}]
	set fill [get opt(-fill) blue]
	set outline [get opt(-outline) black]
	set db $options(-db)
	set source $options(-source)
	if {[info exists opt(-itemcommand)]} {
		proc ::bio::itemcommand {db ft start end} "upvar use use\n$opt(-itemcommand)"
		set itemcommand 1
	} else {
		set itemcommand 0
	}
	# query
	set doquery [list $query and [list _source -> $source] and [list start < $wend] and [list end > $wstart]]
	puts \n-----------------\n$doquery\n-----------------
	puts [time {set fts [$db find -class ft $doquery]}]
	# postfilter
	if {[info exists opt(-postfilter)]} {
		set fts [eval $opt(-postfilter) {$db $fts}]
	}
	# draw
	set y [expr {$ybase-$width*$zoom}]
	set yc [expr {$ybase+$width*$zoom}]
	$w create line $xs $ybase $xe $ybase -tags [list $tag baseline $name] -fill gray -width $baselinewidth
	$w create text [expr {$xs-5}] $ybase -tags [list $tag legend $name] -text $legend -anchor e
	set lastx(0) -1
	set lastx(1) -1
	set shifted(0) 0
	set shifted(1) 0
	set starts {}
	foreach ft $fts {
		lappend starts [$db get $ft start]
	}
	set fts [ssort -integer -reflist $starts $fts]
	foreach ft $fts {
		set ftstart [$db get $ft start]
		set ftend [$db get $ft end]
		set complement [$db get $ft complement 0]
		catch {unset use}
		if {$itemcommand} {
			::bio::itemcommand $db $ft $ftstart $ftend
		}
		set f_labelfont [::bio::zoomfont [get use(font) $labelfont] $zoom]
		set f_linewidth [expr {[get use(linewidth) $linewidth]*$zoom}]
		set xstart [expr {$xs+$xscale*($ftstart-$wstart)}]
		if {$xstart < $xs} {set xstart $xs}
		set xend [expr {$xs+$xscale*($ftend+1-$wstart)}]
		if {$xend > $xe} {set xend $xe}
		if {$shifted($complement) >= $maxshift} {
			set lastx($complement) -1
		}
		if {[catch {$db get $ft _parts} parts]} {
			if {!$complement} {
				if {![info exists use(width)]} {
					set y2 $y
				} else {
					set y2 [expr {$ybase-$use(width)*$zoom}]
				}
				set t +
				set ly [expr {$y2-1}]
				set lastypos 1
				set anchor sw
			} else {
				if {![info exists use(width)]} {
					set y2 $yc
				} else {
					set y2 [expr {$ybase+$use(width)*$zoom}]
				}
				set t -
				set ly [expr {$y2+1}]
				set lastypos 3
				set anchor nw
			}
			$w create rectangle $xstart $ybase $xend $y2 \
				-fill [get use(fill) $fill] -outline [get use(outline) $outline] \
				-width $f_linewidth \
				-tags [list $tag feature $name $ft $t]
			if {$xstart < $lastx($complement)} {
				incr shifted($complement)
				set ly $lasty($complement)
			} else {
				set shifted($complement) 0
			}
			set id [$w create text $xstart $ly \
				-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
				-tags [list $tag label $name $ft $t]]
			set bbox [$w bbox $id]
			set lasty($complement) [lindex $bbox $lastypos]
			if {$shifted($complement) == 0} {
				set lastx($complement) [lindex $bbox 2]
			}
		} else {
			set complement [$db get [lindex $parts 0] complement 0]
			if {!$complement} {
				set uy $y
				set t +
				set ly [expr {$y-1}]
				set anchor s
			} else {
				set uy $yc
				set t -
				set ly [expr {$yc+1}]
				set anchor n
			}
			$w create line $xstart $uy $xend $uy \
				-fill [get use(fill) $fill] \
				-width $f_linewidth \
				-tags [list $tag combofeature $name $ft $t]
			set xpos [expr {$xstart + ($xend-$xstart)/2}]
			set id [$w create text $xpos $ly \
				-text [get use(label) ""] -anchor $anchor -font $f_labelfont \
				-tags [list $tag label $name $ft $t]]
		}
	}
	eval $w raise combofeature all
}

bio::Map method redraw {} {
	private $object options redraw filter filterorder ypos
	if {!$redraw} return
	$object _settings
	set w $options(-canvas)
	$w delete $options(-tag)
	$object draw_axis
	catch {unset ypos}
	set ypos() $options(-yspacing)
	set names [list_union [get filterorder ""] [array names filter]]
	foreach name $names {
		if {[info exists filter($name)]} {
			puts "filter: $name"
			$object draw_filter $name $filter($name)
		}
	}
}

bio::Map method filter {option args} {
	private $object options filter filterorder
	switch $option {
		add {
			set filter([list_shift args]) $args
		}
		delete {
			foreach name $args {
				catch {unset filter($name)}
			}
		}
		order {
			set filterorder [lindex $args 0]
		}
		list {
			return [list_union [get filterorder ""] [array names filter]]
		}
	}
	::Classy::todo $object redraw	
}

bio::Map method printdialog {} {
	private $object options redraw
	if {!$redraw} return
	set w $options(-canvas)
	set page [$w cget -scrollregion]
	set width [lindex $page 2]
	set height [lindex $page 3]
	if {$width < $height} {
		set portrait 1
	} else {
		set portrait 0
	}
	if [winfo exists .classy__.printdialog] {
		destroy .classy__.printdialog
	}
	Classy_printdialog .classy__.printdialog -portrait 0 -colormode color \
		-papersize [list $width $height] \
		-canvas $w -command {}
}

bio::Map method print {args} {
	private $object options redraw
	if {!$redraw} return
	set w $options(-canvas)
	set page [$w cget -scrollregion]
	set width [lindex $page 2]
	set height [lindex $page 3]
	array set print [list -colormode color -printwidth $width -printheight $height]
	array set print $args
	eval ::Classy::printcanvas $w [array get print]
}

