package require dbi
package require dbi_sqlite3
package require Class
package require Compress
namespace eval biodb {}

#auto_index Biodb-sqlite
if {![string length [info commands Biodb-sqlite]]} {
	Biodb subclass Biodb-sqlite
}

Biodb-sqlite method create_backend {database metaVar} {
	private $object db
	upvar $metaVar meta
	dbi_sqlite3 $db
	$db create $database
	$db open $database
	$object initbiodb $database $metaVar
	$db close
	$db destroy
}

Biodb-sqlite method initbiodb {database metaVar} {
	private $object db
	upvar $metaVar meta
	$object createtable fileinfo {
		"key" varchar(100)
		"value" varchar(1000)
	}
	$object insert fileinfo key type value biodb
	$object insert fileinfo key version value $meta(version)
	$object insert fileinfo key dbtype value $meta(-type)
	$object createtable {class} {
		"id" varchar(32) not null primary key
		"superclass" varchar(32)
		"def" varchar(10000)
	}
	$object createtable object {
		"id" int not null primary key
		"class" varchar(32) not null
	}
	$db serial add object id
	# classes
	# ---
	::biodb::class_create_all $object
	$object _updatesuperview
	$db commit
}

Biodb-sqlite method open_backend {database metaVar} {
	private $object db meta
	upvar $metaVar meta
	dbi_sqlite3 $db
	$db open $database
	set type [$db exec {select "value" from "fileinfo" where "key" = 'type'}]
	if {![string equal $type biodb]} {
		$db close $database
		error "not a biodb database: wrong fileinfo"
	}
	set meta(version) [$db exec {select "value" from "fileinfo" where "key" = 'version'}]
}

Biodb-sqlite method insert {args} {
	private $object db
	$db insert [lindex $args 0] [lrange $args 1 end]
}

Biodb-sqlite method set {args} {
	private $object db setfilter
	if {[lindex $args 0] eq "-nullvalue"} {
		set nullvalue [lindex $args 1]
		set args [lrange $args 2 end]
	}
	if {[llength $args] == 2} {
		foreach {id data} $args break
	} else {
		set id [lindex $args 0]
		set data [lrange $args 1 end]
	}
	set table [lindex $id 0]
	$setfilter prefixfilter $table, data
	if {[info exists nullvalue]} {
		return [$db set -nullvalue $nullvalue $id $data]
	} else {
		return [$db set $id $data]
	}
}

Biodb-sqlite method get {args} {
	private $object db getfilter
	set id [lindex $args 0]
	set table [lindex $id 0]
	if {[llength $args] == 1} {
		set data [$db get $id]
		$getfilter prefixfilter $table, data
	} elseif {[llength $args] == 2} {
		set fields [lindex $args 1]
		set data [$db get $id $fields]
		$getfilter prefixfilter $table, data $fields
	} else {
		set fields [lrange $args 1 end]
		set data [$db get $id $fields]
		$getfilter prefixfilter $table, data $fields
	}
	return $data
}

Biodb-sqlite method unset {args} {
	private $object db
	return [eval {$db unset} $args]
	set fullid [lindex $args 0]
	if {[llength $args] > 2} {
		set fields [lrange $args 1 end]
	} else {
		set fields [lindex $args 1]
	}
	$db unset $fullid $fields
}

Biodb-sqlite method new {table args} {
	private $object db
	set id [$db serial next $table id]
	set fullid [list $table $id]
	$db set [list object $id] class $table
	if {[llength $args] == 1} {set args [lindex $args 0]}
	lappend args id $id
	$db set $fullid $args
	return $fullid
}

Biodb-sqlite method createtable {table def} {
	private $object db
	regsub -all \n [string trimright [string trimleft $def "\n\t "] "\n\t "] ,\n def
	set sql [subst {create table "$table" ($def)}]
	$db exec $sql
}

Biodb-sqlite method createclass {superclass table def} {
	private $object db setfilter getfilter
	regsub -all \t\t+ $def \t def
	foreach {field code data} [$setfilter listfull $superclass,*] {
		$setfilter set $table,$field $code $data
	}
	foreach {field code data} [$getfilter listfull $superclass,*] {
		$getfilter set $table,$field $code $data
	}
	foreach line [split $def \n] {
		if {[regexp {^[\t ]*"?([^ \t"]+)"?[\t ]+[^ \t]+[\t ]+references[\t ]+"?([^ \t"]+)} $line temp srcfield desttable]} {
			$object link $table $srcfield $desttable
		}
	}
	if {![string equal $superclass object]} {
		set temp [$db exec -flat {select "id","superclass","def" from "class" where "id" = ?} $superclass]
		if {![llength $temp]} {error "superclass \"$superclass\" not found"}
		foreach {id supersuperclass superdef} $temp break
		set def [string trimright $superdef]\n\t[string trimleft $def]
	}
	regsub -all \n [string trimright [string trimleft $def "\n\t "] "\n\t "] ,\n udef
	set sql [subst {create table "$table" (\n\t"id" int not null primary key,\n\t$udef)}]
	$db begin
	if {[catch {
		$db exec $sql
		$db serial share $table id object id
		$db exec -nullvalue NULL {insert into "class" ("id","superclass","def") values (?,?,?)} $table $superclass $def
	} error]} {
		$db rollback
		error $error
	}
	$db commit
	if {![string equal $superclass object]} {
		$object _superview $superclass
	}
}

Biodb-sqlite method _updatesuperview {} {
	private $object db
	set list [$db exec -flat {select "id" from "class"}]
	set sql [subst {create view "object*" as\nselect "id","class" from "object"}]
	catch {$db exec "drop view \"object*\""}
	$db exec $sql
	set superclasses [list_remove [$db exec -flat {select distinct("superclass") from "class" where "superclass" is not null}] object]
	foreach superclass $superclasses {
		$object _superview $superclass
	}
}

Biodb-sqlite method _superview {superclass} {
	private $object db meta
	set fields [$db fields $superclass]
	set sql [subst {create view "${superclass}*" as\n}]
	append sql [subst {select '$superclass' as "class"}]
	foreach field $fields {
		append sql [subst { ,"$field" as "$field"}]
	}
	append sql [subst {\nfrom "$superclass"}]
	foreach subclass [$object subclasses $superclass] {
		append sql [subst {\nunion\nselect '$subclass' as "class"}]
		foreach field $fields {
			append sql [subst { ,"$field" as "$field"}]
		}
		append sql [subst {\nfrom "$subclass"}]
	}
	catch {$db exec "drop view \"${superclass}*\""}
	$db exec $sql
}

Biodb-sqlite method index {table args} {
	private $object db
	foreach field [list_remove $args id] {
		$db exec [subst {create index "$table-$field" on "$table" ("$field")}]
	}
}

Biodb-sqlite method compress {data} {
	return [compress $data]
}

Biodb-sqlite method uncompress {data} {
	return [uncompress $data]
}
