proc bio_blast {args} {
	cmd_args bio_blast {
		-databases {any "list of databases"}
		-databasedir {dir "directory containing databases"}
	} "sequence" $args
	set blast $::bio::external(blast)/blastall
	set tempsrc [tempfile]
	set tempdst [tempfile]
	set f [open $tempsrc w]
	puts $f ">work"
	puts $f $sequence
	close $f
	set keep [pwd]
	cd $opt(-databasedir)
	catch {exec $blast -p blastn -F F -d $opt(-databases) -i $tempsrc -o $tempdst} result
	cd $keep
}

proc biodb_import_blast_entry {line} {
	upvar ::bio::import_blast import_blast
	set biodb $import_blast(biodb)
	set transmax $import_blast(transmax)
	set eventid $import_blast(eventid)
	catch {unset ta}
	array set ta $line
	set query_name $ta(query_name)
	set ft [$biodb new blast]
	lappend import_blast(ids,$query_name) [lindex $ft 1]
	foreach key {start end hit_start hit_end} {incr ta($key) -1}
	if {![info exists ta(gaps)]} {set ta(gaps) 0}
	log "$ft $query_name $ta(start)-$ta(end)"
	set extra {}
	foreach key {hit_locus hit_acc hit_gi hit_db hit_div hit_version} {
		if {![string equal $ta($key) {}]} {
			lappend extra $key $ta($key)
		}
	}
	if {[catch {
		$biodb set -nullvalue {} $ft event $eventid ftkey similarity \
			start $ta(start) end $ta(end) complement $ta(complement) \
			hit_full_length $ta(hit_full_length) \
			hit_start $ta(hit_start) hit_end $ta(hit_end) hit_complement $ta(hit_complement) \
			bit_score $ta(bit_score) expect $ta(expect) blast_score $ta(score) \
			gaps $ta(gaps) similarity $ta(similarity) identities $ta(identities) \
			hit_size $ta(hit_size) hit_id $ta(hit_id) hit_def [string range $ta(hit_def) 0 249] \
			hit_locus $ta(hit_locus) hit_acc $ta(hit_acc) hit_gi $ta(hit_gi) hit_db $ta(hit_db) \
			hit_div $ta(hit_div) hit_version $ta(hit_version)
	} error]} {
		error "$error\n while inserting \"[array get ta]\""
	}
	incr import_blast(num)
	if {$transmax && ($import_blast(num) >= $transmax)} {
		log "committing after $transmax inserts"
		set import_blast(num) 0
		$biodb commit
		$biodb begin
	}
}

proc biodb_import_blast {args} {
	upvar #0 ::bio::import_blast import_blast
	unset -nocomplain import_blast
	set opt(-transmax) 0
	set opt(-maxexpect) 10
	cmd_args bio_blast {
		-seq {integer "id(s) of sequence in database which was used as a query for the given blast result files"}
		-transmax {integer "commit if more than integer inserts occurred"}
		-maxexpect {integer "maximum expect value (default 10)"}
	} {biodb file} $args
	set transmax $opt(-transmax)
	set import_blast(transmax) $transmax
	set import_blast(biodb) $biodb
	$biodb begin
	if [catch {
		log "Importing file \"$file\""
		if {[file exists $file.runinfo]} {
			set c [file_read $file.runinfo]
		} else {
			set c {}
		}
		set eventid [$biodb event type import sourcetype blastfile source $file options [list_concat $c $args]]
		set import_blast(eventid) [lindex $eventid end]
		set import_blast(num) 0
		set f [open $file]
		set data [blast_parse -maxexpect $opt(-maxexpect) -cmd biodb_import_blast_entry $f info]
		foreach {run_descr run_info} $info optid [get opt(-seq) ""] {
			catch {unset ra}
			array set ra $run_info
			set extra {}
			if {[info exists ra(filter)]} {lappend extra filter $ra(filter)}
			if {[info exists ra(matrix)]} {lappend extra matrix $ra(matrix)}
			if {[isint $optid]} {
				set seq_a($run_descr) $optid
			} else {
				if [catch {$biodb scanid $run_descr} seqid] {
					if [catch {$biodb scanid [file tail [file root [file root $file]]]} seqid] {
						set seqid [$biodb scanid $file]
					}
				}
				set seq_a($run_descr) $seqid
			}
			$biodb set $eventid options [list_concat [list	blast_program $ra(blast_program) \
				blast_version $ra(blast_version) \
				databases $ra(databases) \
				gap_open $ra(gap_open) \
				gap_extend $ra(gap_extend) \
				database_length $ra(database_length) \
				effective_search_space_used $ra(effective_search_space_used) \
				gapped_lambda $ra(gapped_lambda) \
				gapped_K $ra(gapped_K) \
				gapped_M $ra(gapped_M)] $c $extra]
		}
		foreach query_name [array names seq_a] {
			set seqid $seq_a($query_name)
			foreach id [get import_blast(ids,$query_name) {}] {
				$biodb set [list blast $id] seq [lindex $seqid end]
			}
			unset -nocomplain import_blast(ids,$query_name)
		}
	} result] {
		$biodb rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$biodb commit
	}
}

proc blast_run {source dest databases args} {
	set opt(-method) net
	array set opt $args
	set blast $::bio::external($opt(-method)blast)
	set destroot [file root $dest]
	set destext [file extension $dest]
	foreach db $databases {
		set dest $destroot.$db$destext
		set f [open [file join $dest.runinfo] w]
		puts $f [list source $source]
		puts $f [list database $database]
		puts $f [list method $opt(-method)]
		puts $f [list options {}]
		puts $f [list time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]]
		close $f
		exec $blast -p blastx -i $source -o $dest
	}
	file copy $source $dest/$source
	exec $repeatmasker $dest/$source
	file delete $dest/$source
}

