array set conv_blast_seqdb_trans {
	Matrix matrix
	{Number of Hits to DB} hits
	{Number of Sequences} hit_sequences
	{Number of extensions} extensions
	{Number of successful extensions} successfull_extensions
	{Number of sequences better than 10.0} better_than_10
	{Number of HSP's better than 10.0 without gapping} hsps_better_than_10
	{Number of HSP's successfully gapped in prelim test} hsps_gapped_prelim
	{Number of HSP's that attempted gapping in prelim test} hsps_gap_attempt
	{Number of HSP's gapped (non-prelim)} hsps_gapped
	{length of query} query_length
	{length of database} database_length
	{effective HSP length} effective_HSP_length
	{effective length of query} effective_query_length
	{effective length of database} effective_database_length
	{effective search space} effective_search_space
	{effective search space used} effective_search_space_used
}

proc blast_parseinfo {blastinfo blast_program} {
	upvar #0 conv_blast_seqdb_trans trans
	upvar query_name query_name
	upvar query_length query_length
	set info {}
	set pos 0
	set len [llength $blastinfo]
	set info [list blast_program [lindex $blast_program 0] blast_version [lindex $blast_program 1]]
	set line [lindex $blastinfo $pos]
	catch {unset databases}
	while 1 {
		if [regexp {^ *([^:]+): +(.+)$} $line temp key value] {
			switch $key {
				Database {
					set currentdatabase $value
				}
				{Posted date} {
					lappend databases($currentdatabase) date [time_format [time_scan $value]]
				}
				{Number of letters in database} {
					regsub -all , $value {} value
					lappend databases($currentdatabase) letters $value
				}
				{Number of sequences in database} {
					regsub -all , $value {} value
					lappend databases($currentdatabase) sequences $value
				}
				default {
					break
				}
			}
		} else {
			if {[llength $line]} break
		}
		incr pos
		if {$pos >= $len} break
		set line [lindex $blastinfo $pos]
	}
	lappend info query_name $query_name query_length $query_length databases [array get databases]
	while 1 {
		if [regexp {^ *([^:]+): +(.+)$} $line temp key value] {
			if [info exists trans($key)] {
				regsub -all , $value {} value
				lappend info $trans($key) $value
			} elseif {[string equal {frameshift window, decay const} $key]} {
				regsub -all , $value {} value
				lappend info frameshift_window [lindex $value 0] \
					decay_const [lindex $value 1]
			} elseif {[string equal {Gap Penalties} $key]} {
				regsub -all {[^0-9]} $value { } value
				lappend info gap_open [lindex $value 0] \
					gap_extend [lindex $value 1]
			} else {
				regsub -all {[()]} $key {-} key
				lappend info $key $value
			}
		} elseif [regexp {^Gapped} $line] {
			incr pos 2
			set line [lindex $blastinfo $pos]
			lappend info gapped_lambda [lindex $line 0] gapped_K [lindex $line 1] gapped_M [lindex $line 2]
		} elseif [regexp {^Lambda} $line] {
			incr pos
			set line [lindex $blastinfo $pos]
			lappend info lambda [lindex $line 0] K [lindex $line 1] M [lindex $line 2]
		}
		incr pos
		if {$pos >= $len} break
		set line [lindex $blastinfo $pos]
	}
	return $info
}

proc blast_parsequery {channel} {
	while {![eof $channel]} {
		set line [gets $channel]
		if {[string match Query* $line]} {
			set query_name [string range $line 7 end]
			set line [gets $channel]
			while {![eof $channel]} {
				if [regexp {\([0-9,]+} $line query_length] break
				append query_name " $line"
				set line [gets $channel]
			}
			regsub -all , [string range $query_length 1 end] {} query_length
		}
		if [regexp {^>} $line] break
	}
	if {![info exists query_name]} {
		return -code break
	}
	return [list $query_name $query_length $line]
}

proc blast_parse_old {channel infovar cmd maxexpect} {
	if {[string equal maxexpect ""]} {set maxexpect 100000.0}
	if {[string equal $cmd ""]} {set usecmd 0} else {set usecmd 1}
	seek $channel 0
	if {![string_equal $infovar ""]} {
		upvar $infovar info
		set info {}
	}
	set result ""
	while {![eof $channel]} {
		set pos [tell $channel]
		set line [gets $channel]
		regexp {^([^ ]*BLAST[^ ]+ +([0-9.]+))} $line temp blast_program
		if {[string match "Query=*" $line]} {
			if {![info exists blast_program]} {set blast_program bl2seq}
			break
		}
	}
	seek $channel $pos
	if {[eof $channel]} {error "data not recognised as BLAST output"}
	set blast_program [string tolower $blast_program]
	foreach {query_name query_length line} [blast_parsequery $channel] break
#	set pos [string first " " $line]
#	set hit_id [string range $line 1 [expr {$pos-1}]]
#	set hit_def [string range [string trimright $line] [expr {$pos+1}] end]
	set fresult ""
	set blastinfo [list]
	set hit_div {}
	while 1 {
		set id [string range $line 0 2]
		# stop previous
		switch -glob $id {
			{ Sc} - {  D} - {Lam} - >* {
				if {[info exists query]} {
					lappend result end $query_end
					if {$hit_complement != $complement} {set dir -1} else {set dir 1}
					lappend result complement $complement hit_complement 0 dir $dir
					if {$complement} {
						lappend result hit_start $hit_end hit_end $hit_first
					} else {
						lappend result hit_start $hit_first hit_end $hit_end
					}
					lappend result query $query hit $hit
					if {$usecmd} {
						uplevel $cmd [list $result]
					} else {
						lappend fresult $result
					}
					catch {unset complement}
					catch {unset frame}
					catch {unset query}
					catch {unset hit}
				}
			}
		}
		switch -glob $id {
			Que {
				if {![info exists query]} {
					scan $line "Query: %d" start
					set query ""
					lappend result start $start
				}
				append query [lindex $line 2]
				set query_end [lindex $line 3]
			}
			Sbj {
				if {![info exists hit]} {
					scan $line "Sbjct: %d" temp
					set hit ""
					set hit_first $temp
				}
				append hit [lindex $line 2]
				set hit_end [lindex $line 3]
			}
			{  D} - {Lam} {
				while {![eof $channel]} {
					lappend blastinfo $line
					set line [gets $channel]
					if {[string match S1:* $line]} break
				}
				lappend blastinfo $line
				if {![string_equal $infovar ""]} {
					lappend info $query_name [blast_parseinfo	$blastinfo $blast_program]
				}
				foreach {query_name query_length line} [blast_parsequery $channel] break
				set blastinfo [list]
				# we still need to check this line
				continue
			}
			{ [A-Za-z]*} {
				foreach sub [split $line ,] {
					foreach {key value} [split $sub =] break
					switch -regexp $key {
						{Score} {
							scan $value " %f bits (%d)" bit_score score
							set result [list query_name $query_name \
								hit_id $hit_id hit_def $hit_def hit_acc $hit_acc hit_version $hit_version\
								hit_locus $hit_locus hit_gi $hit_gi hit_db $hit_db hit_div $hit_div\
								hit_full_length $hit_full_length score $score bit_score $bit_score]
						}
						{Expect} {
							set expect [string range $value 1 end]
							regsub ^e $expect 1e expect
							lappend result expect $expect
						}
						{Identities} {
							scan $value " %d/%d (%d%%)" identities hit_size similarity
							lappend result identities $identities hit_size $hit_size similarity $similarity
						}
						{Positives} {
							scan $value " %d" positives
							lappend result positives $positives
						}
						{Gaps} {
							scan $value " %d" gaps
							lappend result gaps $gaps
						}
						{Frame} {
							set frame [string range $value 1 end]
							lappend result frame $frame
							if {$frame > 0} {
								set complement 0
							} else {
								set complement 1
							}
						}
						{Strand} {
							if {[string_equal [string index $value 8] M]} {
								set complement 1
							} else {
								set complement 0
							}
						}
					}
				}
			}
			>* {
				set pos [string first " " $line]
				set hit_id [string range $line 1 [expr {$pos-1}]]
				set hit_def [string range [string trimright $line] [expr {$pos+1}] end]
				foreach {hit_gi hit_db hit_acc hit_version hit_locus} [blast_parse_id $hit_id] break
				while {![eof $channel]} {
					set line [gets $channel]
					if [regexp {Length = ([0-9]+)$} $line temp hit_full_length] break
					append hit_def " [string trimleft $line]"
				}
			}
		}
		set line [gets $channel]
		if {[eof $channel]} {
			if {![string_equal $infovar ""]} {
				lappend info $query_name [blast_parseinfo	$blastinfo $blast_program]
			}
			break
		}
	}
	close $channel
	return $fresult
}

proc blast_parse_id {id} {
	set list [split $id |]
	if {[string equal [lindex $list 0] gi]} {
		set gi [lindex $list 1]
		set list [lrange $list 2 end]
	} else {
		set gi {}
	}
	set db {}
	set acc {}
	set version {}
	set locus {}
	foreach {db acc locus} $list break
	regexp {^(.*)\.([0-9]+)$} $acc temp acc version
	switch $db {
		gnl {
			set db $acc
			set acc $locus
		}
		pir - prf {
			set acc $locus
		}
		pat {
			set temp $acc
			set acc $locus
			set locus $acc
		}
		lcl {
			set acc $locus
		}
	}
	return [list $gi $db $acc $version $locus]
}

proc blast_parse_xml {channel infovar cmd maxexpect} {
	if {[string equal maxexpect ""]} {set maxexpect 100000.0}
	if {[string equal $cmd ""]} {set usecmd 0} else {set usecmd 1}
	seek $channel 0
	if {![string_equal $infovar ""]} {
		upvar $infovar info
		set info {}
	}
	set result ""
	set info {}
	catch {unset ta}
	set hit_div {}
	while {![eof $channel]} {
		set line [gets $channel]
		if {[regexp {<([^>]+)>([^<]+)<([^>]+)>} $line temp key value]} {
			set ta($key) $value
		} elseif {[regexp {</BlastOutput_param>} $line]} {
			set hit_div [file root [file tail $ta(BlastOutput_db)]]
		} elseif {[regexp {</BlastOutput>} $line]} {
			set blastentry [list blast_version [lindex $ta(BlastOutput_version) 1]]
			foreach {key bkey} {
				blast_program BlastOutput_program
				databases BlastOutput_db
				query_name BlastOutput_query-def
				query_length BlastOutput_query-len
				matrix Parameters_matrix
				score_match Parameters_sc-match
				score_mismatch Parameters_sc-mismatch
				gap_open Parameters_gap-open
				gap_extend Parameters_gap-extend
				filter Parameters_filter
				database_number Statistics_db-num
				database_length Statistics_db-len
				hsp_length Statistics_hsp-len
				effective_search_space_used Statistics_eff-space
				gapped_K Statistics_kappa
				gapped_lambda Statistics_lambda
				gapped_M Statistics_entropy
			} {
				if {[info exists ta($bkey)]} {
					lappend blastentry $key $ta($bkey)
				}
			}
			lappend info $ta(BlastOutput_query-def) $blastentry
		} elseif {[regexp {</Hsp>} $line]} {
			if {$ta(Hsp_evalue) > $maxexpect} continue
			if {![info exists ta(Hsp_hit-frame)] || ($ta(Hsp_hit-frame) == 0)} {
				# DNA - protein hit
				if {$ta(Hsp_query-frame) < 0} {set complement 1} else {set complement 0}
				set hit_complement 0
			} elseif {$ta(Hsp_query-frame) < 0} {
				if {$ta(Hsp_hit-frame) < 0} {set complement 0} else {set complement 1}
				set hit_complement 1
			} else {
				if {$ta(Hsp_hit-frame) < 0} {set complement 1} else {set complement 0}
				set hit_complement 0
			}
			if {$hit_complement != $complement} {set dir -1} else {set dir 1}
			if {$ta(Hsp_hit-from) > $ta(Hsp_hit-to)} {
				set temp $ta(Hsp_hit-from)
				set ta(Hsp_hit-from) $ta(Hsp_hit-to)
				set ta(Hsp_hit-to) $temp
			}
			if {$ta(Hsp_query-from) > $ta(Hsp_query-to)} {
				set temp $ta(Hsp_query-from)
				set ta(Hsp_query-from) $ta(Hsp_query-to)
				set ta(Hsp_query-to) $temp
			}
			foreach {hit_gi hit_db hit_acc hit_version hit_locus} [blast_parse_id $ta(Hit_id)] break
			set hitentry [list \
				hit_db $hit_db \
				hit_div $hit_div \
				hit_gi $hit_gi \
				hit_acc $hit_acc \
				hit_version $hit_version \
				hit_locus $hit_locus \
				complement $complement \
				hit_complement $hit_complement \
				dir $dir]
			if {[info exists ta(Hsp_identity)] && [info exists ta(Hsp_align-len)]} {
				lappend hitentry similarity [expr {100*double($ta(Hsp_identity))/$ta(Hsp_align-len)}]
			}
			if {[info exists ta(Iteration_query-def)]} {
				lappend hitentry query_name $ta(Iteration_query-def)
			} elseif {[info exists ta(BlastOutput_query-def)]} {
				lappend hitentry query_name $ta(BlastOutput_query-def)
			}
			foreach {key bkey} {
				hit_id Hit_id
				hit_def Hit_def
				hit_full_length Hit_len
				score Hsp_score
				bit_score Hsp_bit-score
				expect Hsp_evalue
				identities Hsp_identity
				positives Hsp_positive
				gaps Hsp_gaps
				align_length Hsp_align-len
				hit_size Hsp_align-len
				frame Hsp_query-frame
				hit_frame Hsp_hit-frame
				start Hsp_query-from
				end Hsp_query-to
				hit_start Hsp_hit-from
				hit_end Hsp_hit-to
				pattern_start Hsp_pattern-from
				pattern_end Hsp_pattern-to
				query Hsp_qseq
				hit Hsp_hseq
				midline Hsp_midline
				density Hsp_density
			} {
				if {[info exists ta($bkey)]} {
					lappend hitentry $key $ta($bkey)
				}
			}
			if {$usecmd} {
				uplevel $cmd [list $hitentry]
			} else {
				lappend result $hitentry
			}
		}
	}
	close $channel
	return $result
}

proc blast_parse {args} {
	set opt(-cmd) {}
	set opt(-maxexpect) 10
	cmd_args bio_blast {
		-cmd {string "command to execute with each entry as an argument"}
		-maxexpect {integer "maximum expect value (default 10)"}
	} {channel ?infovar?} $args
	if {[info exists infovar]} {
		upvar $infovar info
	}
	set line [gets $channel]
	if {[string match *<* $line]} {
		if {[string match "*<!DOCTYPE BlastOutput*" $line]} {
			set fileformat xml
		} else {
			set line [gets $channel]
			if {[string match "*<!DOCTYPE BlastOutput*" $line]} {
				set fileformat xml
			}
		}
	} else {
		while {![eof $channel]} {
			if {[string match "Query=*" $line]} {
				set fileformat old
				break
			}
			set line [gets $channel]
		}
	}
	if {![info exists fileformat]} {
		error "data not recognised as BLAST output"
	}
	blast_parse_$fileformat $channel info $opt(-cmd) $opt(-maxexpect)
}
