proc parse_fasta {file} {
	set f [open $file]
	while 1 {
		set line [gets $f]
		if {([string equal [string index $line 0] >]) || ([eof $f])} {
			if {[eof $f]} break
			set seqname [string range $line 1 end]
		} elseif {[string equal $line ""]} {
		} else {
			if {![info exists seqname]} {error "error in format"}
			append result($seqname) $line
		}
	}
	return [array get result]
}

proc conv_fasta_biodb {args} {
putsvars args
	cmd_args conv_embl_seqdb {
		-log {channel "channel on which to write logs"}
		-name {string "sequence name"}
		-acc {string "sequence accession"}
	} {db file} $args
	catch {set log $opt(-log)}
	$db begin
	if [catch {
		set f [open $file]
		set eventid [$db new event]
		$db set $eventid type import sourcetype fastafile source $file
		set length 0
		set goal 100000
		set buffer {}
		while 1 {
			set line [gets $f]
			if {([string equal [string index $line 0] >]) || ([eof $f])} {
				if {[info exists seqid]} {
					incr length [string length $line]
					$db sequence append $seqid $buffer$line
					$db set $seqid length $length
					set length 0
				}
				if {[eof $f]} break
				set seqname [string range $line 1 end]
				if {[info exists log]} {puts $log $seqname}
				set seqid [$db new seq]
				set temp [split $seqname |]
				if {[llength $temp] >= 4} {
					foreach {acc version} [split [lindex $temp 3] .] break
					set name [join [lrange $temp 0 1] |]
				} else {
					set acc [string range $seqname 0 14]
					set version {}
					set name $acc
				}
				if {[info exists opt(-acc)]} {
					set acc $opt(-acc)
				}
				if {[info exists opt(-name)]} {
					set name $opt(-name)
				}
				$db set -nullvalue {} $seqid event [lindex $eventid 1] name $name comment $seqname acc $acc version $version
				$db sequence set $seqid {}
			} elseif {[string equal $line ""]} {
			} else {
				if {![info exists seqid]} {
					error "error in format"
				}
				incr length [string length $line]
				append buffer $line
				if {$length > $goal} {
					$db sequence append $seqid $buffer
					set buffer {}
					set goal [expr {$length + 100000}]
				}
				#puts -nonewline $o $line
			}
		}
		close $f
	} result] {
		$db rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$db commit
	}
}

proc conv_biodb_fasta {args} {
	cmd_args conv_embl_seqdb {
		-separate {switch "create separate files"}
	} {db ids file} $args
	if {[info exists opt(-separate)]} {set sep 1} else {set sep 0}
	if {!$sep} {
		set f [open $file w]
	} else {
		set root [file root $file]
		set ext [file extension $file]
	}
	foreach id $ids {
		log "$id"
		foreach {acc name} [$db get $id acc name] break
		if {$sep} {set f [open ${root}_${id}_$acc$ext w]}
		puts $f ">[lindex $id 1]_$acc $name"
		$db sequence dump $id $f
		puts $f ""
		if {$sep} {close $f}
	}
	if {!$sep} {
		close $f
	}
}
