proc bio_fasta_clean {source dest} {
	set f [open $source]
	set o [open $dest w]
	set line [gets $f]
	puts $o $line
	while {![eof $f]} {
		set line [gets $f]
		if {![string length $line]} continue
		if {[string equal [string index $line 0] >]} {
			puts $o \n$line
		} else {
			puts -nonewline $o $line
		}
	}
	close $o
	close $f
}

proc bio_fasta_filesplit {source dest} {
	set f [open $source]
	set destroot [file root $dest]
	set destext [file extension $dest]
	set num 1
	while {![eof $f]} {
		set line [gets $f]
		if {![string length $line]} continue
		if {[string equal [string index $line 0] >]} {
			catch {close $o}
			set id [lindex [string range $line 1 end] 0]
			set o [open ${destroot}${num}_$id$destext w]
			incr num
			puts $o $line
		} else {
			puts -nonewline $o $line
		}
	}
	catch {close $o}
	close $f
}

proc bio_cfasta_split {source dest size overlap} {
	set f [open $source]
	fconfigure $f -buffering line
	set o [open $dest w]
	set lsize [expr {$size-$overlap}]
	set num 1
	set start 0
	set end $size
	set header [gets $f]
	set pre [nread $f $overlap]
	while {1} {
		if {[neof $f]} break
		set line [nread $f $lsize]
		puts $o $header_$start_$end
		puts $o $pre$line
		incr start $size
		incr end $size
		if {[string length $line] != $lsize} {
			set header [nread $f]
			if {[neof $f]} break
			set pre [nread $f $overlap]
			set num 1
		} else {
			incr num
			set pre [string range $line end-$overlap end]
		}
	}
	close $o
	close $f
}

proc bio_cfasta_splitf {source dest size overlap} {
	set droot [file root $dest]
	set dext [file ext $dest]
	set f [open $source]
	set lsize [expr {$size-$overlap}]
	set num 1
	set header [gets $f]
	set pre [nread $f $overlap]
	while {1} {
		if {[neof $f]} break
		set line [nread $f $lsize]
		set o [open $droot-$num$dext w]
		puts $o $header$num
		puts $o $pre$line
		close $o
		if {[string length $line] != $lsize} {
			set header [nread $f]
			if {[neof $f]} break
			set pre [nread $f $overlap]
			set num 1
		} else {
			incr num
			set pre [string range $line end-$overlap end]
		}
	}
	close $f
}

proc bio_fasta_split {source dest size overlap} {
	set f [open $source]
	set o [open $dest w]
	set prepos [expr {$size-$overlap}]
	incr size -1
	set start 0
	set seq {}
	set cursize 0
	while {![eof $f]} {
		set line [gets $f]
		if {[string equal [string index $line 0] >]} {
			if {$cursize > $overlap} {
				puts $o ${header}_$start
				puts $o $seq
			}
			set start 0
			set cursize 0
			set seq {}
			set header $line
		} else {
			append seq $line
			incr cursize [string length $line]
			while {$cursize > $size} {
				puts $o ${header}_$start
				#puts $o [string range $seq 0 $size]
				set tseq [string range $seq 0 $size]
				for {set i 0} {$i < $size} {incr i 50} {
					puts $o [string range $tseq $i [expr {$i+49}]]
				}
				incr start $prepos
				set seq [string range $seq $prepos end]
				incr cursize -$prepos
			}
		}
	}
	if {$cursize > $overlap} {
		puts $o ${header}_$start
#		puts $o $seq
		for {set i 0} {$i < $cursize} {incr i 50} {
			puts $o [string range $seq $i [expr {$i+49}]]
		}
	}
	close $o
	close $f
}

