proc fasta_parse_old {channel infovar cmd maxexpect} {
	if {[string equal maxexpect ""]} {set maxexpect 100000.0}
	if {[string equal $cmd ""]} {set usecmd 0} else {set usecmd 1}
	if {![string_equal $infovar ""]} {
		upvar $infovar info
	}
	set result ""
	seek $channel 0
	set line [gets $channel]
	set line [gets $channel]
	while {![eof $channel]} {
		set line [gets $channel]
		if {[string match *FASTA* $line]} break
	}
	set cline $line
	while {![eof $channel]} {
		set line [gets $channel]
		append cline $line
		if {[string match "*best scores*" $line]} break
	}
	regexp {ktup: *([0-9]+).*join: *([0-9]+).*opt: *([0-9]+).*gap-pen: *([0-9/-]+).*width: *([0-9]+)} $cline temp infoa(ktup) infoa(join) infoa(opt) infoa(gap-pen) infoa(width)
	set info [array get infoa]
	while {![eof $channel]} {
		set pos [tell $channel]
		set line [gets $channel]
		if {[string match >>* $line]} break
	}
	set fresult ""
	set fastainfo [list]
	set hit_div {}
	while 1 {
		unset -nocomplain a
		set pos [string first " " $line]
		set a(hit_id) [string range $line 2 [expr {$pos-1}]]
		set a(hit_def) [string range [string trimright $line] [expr {$pos+1}] end]
		regexp {^(.*) \(([0-9]+) nt\)$} $a(hit_def) temp a(hit_def) a(hit_full_length)
		foreach {a(hit_gi) a(hit_db) a(hit_acc) a(hit_version) a(hit_locus)} [fasta_parse_id $a(hit_id)] break
		set cline $line
		# header
		while {![eof $channel]} {
			set line [gets $channel]
			if {[string equal $line ""]} break
			append cline " $line"
		}
		foreach {pattern key} {
			{initn: *([0-9.]+)} initn
			{init1: *([0-9.]+)} init1
			{Z-score: *([0-9.]+)} z_score
			{bits: *([0-9.]+)} bit_score
			{E(): *([0-9.e+-]+)} expect
			{Smith-Waterman score: *([0-9-]+)} sw_score
			{([0-9.]+)% *identity} similarity
		} {
			regexp $pattern $cline temp a($key)
		}
		if {![regexp {\(([0-9]+)-([0-9]+):([0-9]+)-([0-9]+)\)} $cline temp a(start) a(end) a(hit_start) a(hit_end)]} {
			break
		}
		if {$a(start) > $a(end)} {
			set a(complement) 1
			set temp $a(start)
			set a(start) $a(end)
			set a(end) $temp
		} else {
			set a(complement) 0
		}
		# alignment
		set a(qseq) {}
		set a(midline) {}
		set a(hseq) {}
		while {![eof $channel]} {
			set line [gets $channel]
			if {[string equal [string range $line 0 1] >>]} break
			set line [gets $channel]
			append a(qseq) [string range $line 7 end]
			set line [gets $channel]
			append a(midline) [string range $line 7 end]
			set line [gets $channel]
			append a(hseq) [string range $line 7 end]
			set line [gets $channel]
			set line [gets $channel]
		}
		set result [array get a]
		if {$usecmd} {
			uplevel $cmd [list $result]
		} else {
			lappend fresult $result
		}
	}
	close $channel
	return $fresult
}

proc fasta_parse_id {id} {
	set list [split $id |]
	if {[string equal [lindex $list 0] gi]} {
		set gi [lindex $list 1]
		set list [lrange $list 2 end]
	} else {
		set gi {}
	}
	set db {}
	set acc {}
	set version {}
	set locus {}
	foreach {db acc locus} $list break
	regexp {^(.*)\.([0-9]+)$} $acc temp acc version
	switch $db {
		gnl {
			set db $acc
			set acc $locus
		}
		pir - prf {
			set acc $locus
		}
		pat {
			set temp $acc
			set acc $locus
			set locus $acc
		}
		lcl {
			set acc $locus
		}
	}
	return [list $gi $db $acc $version $locus]
}

proc fasta_parse {args} {
	set opt(-cmd) {}
	set opt(-maxexpect) 10
	cmd_args bio_fasta {
		-cmd {string "command to execute with each entry as an argument"}
		-maxexpect {integer "maximum expect value (default 10)"}
	} {channel ?infovar?} $args
	if {[info exists infovar]} {
		upvar $infovar info
	}
	set line [gets $channel]
	set line [gets $channel]
	if {![regexp FASTA $line]} {
		error "data not recognised as fasta output"
	}
	fasta_parse_old $channel info $opt(-cmd) $opt(-maxexpect)
}


