proc genscan_run {source dest args} {
	set opt(-param) HumanIso
	cmd_args bio_genscan_run {
		-param {any "parameters to use (HumanIso, Arabidopsis, Maize) or name of parameter file"}
	} {} $args
	set genscan $::bio::external(genscan)
	set genscandir $::bio::external(genscandir)
	if {[inlist {HumanIso Arabidopsis Maize} $opt(-param)]} {
		set paramfile [file join $genscandir $opt(-param).smat]
	} else {
		if {![file exists $opt(-param)]} {error "file \"$opt(-param)\" does not exists"}
		set paramfile $opt(-param)
	}
	set f [open $dest.runinfo w]
	puts $f [list source $source]
	puts $f [list time [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]]
	puts $f [list paramfile $paramfile]
	close $f
	file mkdir $dest
	bio_fasta_filesplit $source $dest/split.fas
	foreach file [glob $dest/*.fas] {
		log $file
		catch {exec $genscan $paramfile $file > [file root $file].genscan} result
	}
	if ![regexp {done.*Run time ([0-9]+) seconds :  ([0-9.]+) s / kb} $result temp runtime perkb] {
		error "error from genscan: $result"
	}
}

proc bio_genscan {args} {
	set opt(-param) HumanIso
	set opt(-cacheresult) {}
	cmd_args bio_genscan {
		-param {any "parameters to use (HumanIso, Arabidopsis, Maize) or name of parameter file"}
		-cacheresult {filename "save resultfile here"}
	} "db source" $args
	# $db class add genscan
	if {[string length $opt(-cacheresult)]&&[file exists $opt(-cacheresult)]} {
		set tempdst $opt(-cacheresult)
	} else {
		set genscan $::bio::external(genscan)
		set genscandir $::bio::external(genscandir)
		if {[inlist {HumanIso Arabidopsis Maize} $opt(-param)]} {
			set paramfile [file join $genscandir $opt(-param).smat]
		} else {
			if {![file exists $opt(-param)]} {error "file \"$opt(-param)\" does not exists"}
			set paramfile $opt(-param)
		}
		set tempsrc [tempfile]
		set tempdst [tempfile]
		set f [open $tempsrc w]
		puts $f ">work"
		puts $f [$db get $source dnaseq]
		close $f
		catch {exec $genscan $paramfile $tempsrc > $tempdst} result
		if ![regexp {done.*Run time ([0-9]+) seconds :  ([0-9.]+) s / kb} $result temp runtime perkb] {
			error "error from genscan: $result"
		}
		file delete $tempsrc
	}
	set eventid [biodb_import_genscan $db $tempdst]
	$db set $eventid type run source $source
	if {[string length $opt(-cacheresult)]} {
		if {![file exists $opt(-cacheresult)]} {
			file rename $tempdst $opt(-cacheresult)
		}
	} else {
		file delete $tempdst
	}
	return $eventid
}

proc biodb_import_onegenscan {biodb source eventid} {
	set f [open $source]
	while {![eof $f]} {
		set line [gets $f]
		if {[regexp {Date run:[[:space:]]*([^ ]+)[[:space:]]*Time:[[:space:]]*([^ ]+)} $line temp date time]} {
		} elseif {[regexp {Parameter matrix:[[:space:]]([^ ]+)} $line temp matrix]} {
		} elseif {[regexp {^Sequence[[:space:]]([^ ]+) : ([0-9]+)} $line temp seqid size]} {
		} elseif {[regexp -- {--} $line]} break
	}
	if [catch {$biodb scanid $seqid} seqid] {
		set seqid [$biodb scanid $source]
	}
	regsub {1([0-9][0-9])} $date {\1} date
	set line [gets $f]
	set fts {}
	set found 0
	set tot_p_score 0
	set tot_start $size
	set tot_end 0
	while {![eof $f]} {
		set line [gets $f]
		if {[string equal [lindex $line 11] nan]} {
			set line [lreplace $line 11 11 0]
		}
		set len [llength $line]
		if {!$len} {
			set len [llength $fts]
			if {$len > 1} {
				set ft [$biodb new genscan]
				if {[string_equal $strand -]} {
					set fts [list_reverse $fts]
					set complement 1
				} else {
					set complement 0
				}
				$biodb set $ft event [lindex $eventid 1] ftkey CDS \
					seq [lindex $seqid 1] start $tot_start end $tot_end complement $complement \
					p_score [expr {$tot_p_score/$len}]
				$biodb ftparts $ft $fts
				set tot_p_score 0
				set tot_start $size
				set tot_end 0
				set fts {}
				incr found
			}
			continue
		}
		set type [lindex $line 1]
		set p 0
		set ft [$biodb new genscan]
		switch $type {
			Sngl {
				set ftkey CDS
				foreach {id type strand start end} $line break
				set data [list_merge {frame phase score_in score_out score_coding p_score total_score} [lrange $line 6 end]]
			}
			Term - Intr - Init {
				set ftkey exon
				lappend fts $ft
				set ftkey exon
				foreach {id type strand start end} $line break
				set data [list_merge {frame phase score_in score_out score_coding p_score total_score} [lrange $line 6 end]]
				set tot_p_score [expr {$tot_p_score + [lindex $line 11]}]
				if {$tot_start > $start} {set tot_start $start}
				if {$tot_end < $end} {set tot_end $end}
			}
			Prom {
				foreach {id type strand start end length total_score} $line break
				set ftkey promoter
				set data [list total_score $total_score]
			}
			PlyA {
				foreach {id type strand start end length total_score} $line break
				set ftkey polyA_signal
				set data [list total_score $total_score]
			}
			default {
				break
			}
		}
		if {[string_equal $strand -]} {
			set complement 1
		} else {
			set complement 0
		}
		incr start -1
		incr end -1
		eval {$biodb set $ft event [lindex $eventid 1] ftkey $ftkey \
			seq [lindex $seqid 1] start $start end $end complement $complement type $type} $data
		log "$ft $seqid $type $start-$end"
	}
	close $f
}

proc biodb_import_genscan {biodb source} {
	if {[file exists $source.runinfo]} {
		set c [file_read $source.runinfo]
	} else {
		set c {}
	}
	if {[file isdir $source]} {
		set files [glob $source/*.genscan]
	} else {
		set files $source
	}
	$biodb begin
	if [catch {
		set eventid [$biodb event type import sourcetype genscan source $source]
		lappend c event [lindex $eventid 1]
		foreach file $files {
			biodb_import_onegenscan $biodb $file $eventid
		}
	} result] {
		$biodb rollback
		return -code error -errorinfo $::errorInfo $result
	} else {
		$biodb commit
	}
	return $eventid
}

