set bio::ambig_dict {
	A A G G C C U T T T
	R {A G} Y {C T} W {A T} S {C G} M {A C} K {G T}
	B {C G T} D {A G T} H {A C T} V {A C G} N {A C G T}
	* *
	{A G} R {C T} Y {A T} W {C G} S {A C} M {G T} K
	{C G T} B {A G T} D {A C T} H {A C G} V {A C G T} N
}

proc bio::ambig_init {} {
	upvar #0 ::bio::ambig_bits bits
	upvar #0 ::bio::ambig_rbits rbits
	unset -nocomplain bits
	set d $::bio::ambig_dict
	array set tbits {A 1 C 2 G 4 U 8 T 8}
	for {set i 1} {$i < 16} {incr i} {
		set bits($i) $i
		set rbits($i) $i
	}
	foreach {ab bases} $d {
		if {$ab eq "*"} break
		set num 0
		foreach b [dict get $d $ab] {
			incr num $tbits($b)
		}
		set bits($ab) $num
		set rbits($num) $ab
	}
}
bio::ambig_init

proc bio::ambig2bits {base} {
	return $::bio::ambig_bits($base)
}

proc bio::bits2ambig {bits} {
	return $::bio::ambig_rbits($bits)
}

proc bio::ambig_match {b1 b2} {
	expr {[get ::bio::ambig_bits($b1) 0] & [get ::bio::ambig_bits($b2) 0]}
}

proc bio::ambig {base} {
	set temp [lsort [list_remdup [split $base ""]]]
	if {[catch {dict get $::bio::ambig_dict $temp} result]} {
		if {[llength $temp] == 1} {error "unknown ambiguity code $base"}
		set bits [get ::bio::ambig_bits([lindex $temp 0]) 0]
		foreach el [lrange $temp 1 end] {
			set bits [expr {$bits | [get ::bio::ambig_bits($el) 0]}]
		}
		set result $::bio::ambig_rbits($bits)
	}
	return $result
}

proc ali_similarity {seq1 seq2 args} {
	if {[string length $seq1] != [string length $seq2]} {
		error "sequences not properly aligned"
	}
	regsub -all {[^-A-Z]} $seq1 {} seq1
	regsub -all {[^-A-Z]} $seq2 {} seq2
	if {[string length $seq1] != [string length $seq2]} {
		error "sequences not properly aligned"
	}
	set total 0
	set match 0
	if {"$args" == ""} {
		foreach b1 [split $seq1 {}] b2 [split $seq2 {}] {
			if {("$b1" == "-")&&("$b2" == "-")} continue
			incr total
			if {"$b1" == "$b2"} {incr match}
		}
	} elseif {"$args" == "-nogaps"} {
		foreach b1 [split $seq1 {}] b2 [split $seq2 {}] {
			if {("$b1" == "-")||("$b2" == "-")} continue
			incr total
			if {"$b1" == "$b2"} {incr match}
		}
	} elseif {"$args" == "-ref"} {
		foreach b1 [split $seq1 {}] b2 [split $seq2 {}] {
			if {"$b1" == "-"} continue
			incr total
			if {"$b1" == "$b2"} {incr match}
		}
	} elseif {"$args" == "-ambig"} {
		foreach b1 [split $seq1 {}] b2 [split $seq2 {}] {
			if {("$b1" == "-")||("$b2" == "-")} continue
			incr total
			if {[bio::ambig_match $b1 $b2]} {incr match}
		}
	}
	return [expr double($match)/double($total)]
}

proc similarity {seq1 seq2 args} {
	set ali [ali_align $seq1 $seq2]
	eval ali_similarity $ali $args
}
